import re

from . import AWSObject, AWSProperty, Join, Tags
from .validators import boolean, integer, positive_integer

MINIMUM_MEMORY = 128
MAXIMUM_MEMORY = 10240

PACKAGE_TYPES = ["Image", "Zip"]
RESERVED_ENVIRONMENT_VARIABLES = [
    "AWS_ACCESS_KEY",
    "AWS_ACCESS_KEY_ID",
    "AWS_DEFAULT_REGION",
    "AWS_EXECUTION_ENV",
    "AWS_LAMBDA_FUNCTION_MEMORY_SIZE",
    "AWS_LAMBDA_FUNCTION_NAME",
    "AWS_LAMBDA_FUNCTION_VERSION",
    "AWS_LAMBDA_LOG_GROUP_NAME",
    "AWS_LAMBDA_LOG_STREAM_NAME",
    "AWS_REGION",
    "AWS_SECRET_ACCESS_KEY",
    "AWS_SECRET_KEY",
    "AWS_SECURITY_TOKEN",
    "AWS_SESSION_TOKEN",
    "LAMBDA_RUNTIME_DIR",
    "LAMBDA_TASK_ROOT",
    "TZ",
]
ENVIRONMENT_VARIABLES_NAME_PATTERN = r"[a-zA-Z][a-zA-Z0-9_]+"


def validate_memory_size(memory_value):
    """Validate memory size for Lambda Function
    :param memory_value: The memory size specified in the Function
    :return: The provided memory size if it is valid
    """
    memory_value = int(positive_integer(memory_value))
    if not MINIMUM_MEMORY <= memory_value <= MAXIMUM_MEMORY:
        raise ValueError(
            "Lambda Function memory size must be between %d and %d"
            % (MINIMUM_MEMORY, MAXIMUM_MEMORY)
        )
    return memory_value


def validate_package_type(package_type):
    """Validate PackageType for Lambda Function.
    :param package_type: The PackageType specified in the Function.
    :return: The provided package type if it is valid.
    """
    if package_type not in PACKAGE_TYPES:
        raise ValueError(
            "Lambda Function PackageType must be one of: {}".format(
                ", ".join(PACKAGE_TYPES)
            )
        )
    return package_type


def validate_variables_name(variables):
    for name in variables:
        if name in RESERVED_ENVIRONMENT_VARIABLES:
            raise ValueError(
                "Lambda Function environment variables names"
                " can't be none of:\n %s" % ", ".join(RESERVED_ENVIRONMENT_VARIABLES)
            )
        elif not re.match(ENVIRONMENT_VARIABLES_NAME_PATTERN, name):
            raise ValueError("Invalid environment variable name: %s" % name)

    return variables


class Code(AWSProperty):
    props = {
        "ImageUri": (str, False),
        "S3Bucket": (str, False),
        "S3Key": (str, False),
        "S3ObjectVersion": (str, False),
        "ZipFile": (str, False),
    }

    @staticmethod
    def check_zip_file(zip_file):
        maxlength = 4096
        toolong = (
            "ZipFile length cannot exceed %d characters. For larger "
            "source use S3Bucket/S3Key properties instead. "
            "Current length: %d"
        )

        if zip_file is None:
            return

        if isinstance(zip_file, str):
            z_length = len(zip_file)
            if z_length > maxlength:
                raise ValueError(toolong % (maxlength, z_length))
            return

        if isinstance(zip_file, Join):
            # This code tries to combine the length of all the strings in a
            # join. If a part is not a string, we do not count it (length 0).
            delimiter, values = zip_file.data["Fn::Join"]

            # Return if there are no values to join
            if not values or len(values) <= 0:
                return

            # Get the length of the delimiter
            if isinstance(delimiter, str):
                d_length = len(delimiter)
            else:
                d_length = 0

            # Get the length of each value that will be joined
            v_lengths = [len(v) for v in values if isinstance(v, str)]

            # Add all the lengths together
            z_length = sum(v_lengths)
            z_length += (len(values) - 1) * d_length

            if z_length > maxlength:
                raise ValueError(toolong % (maxlength, z_length))
            return

    def validate(self):
        image_uri = self.properties.get("ImageUri")
        zip_file = self.properties.get("ZipFile")
        s3_bucket = self.properties.get("S3Bucket")
        s3_key = self.properties.get("S3Key")
        s3_object_version = self.properties.get("S3ObjectVersion")

        if zip_file and image_uri:
            raise ValueError("You can't specify both 'ImageUri' and 'ZipFile'")
        if zip_file and s3_bucket:
            raise ValueError("You can't specify both 'S3Bucket' and 'ZipFile'")
        if zip_file and s3_key:
            raise ValueError("You can't specify both 'S3Key' and 'ZipFile'")
        if zip_file and s3_object_version:
            raise ValueError("You can't specify both 'S3ObjectVersion' and 'ZipFile'")
        if image_uri and (s3_bucket or s3_key or s3_object_version):
            raise ValueError(
                "You can't specify 'ImageUri' and any of 'S3Bucket', 'S3Key', "
                "or 'S3ObjectVersion' at the same time"
            )
        Code.check_zip_file(zip_file)
        if not zip_file and not (s3_bucket and s3_key) and not image_uri:
            raise ValueError(
                "You must specify a bucket location (both the 'S3Bucket' and "
                "'S3Key' properties), the 'ImageUri' property, "
                "or the 'ZipFile' property"
            )


class ImageConfig(AWSProperty):
    props = {
        "Command": ([str], False),
        "EntryPoint": ([str], False),
        "WorkingDirectory": (str, False),
    }

    def validate(self):
        command = self.properties.get("Command")
        if command and len(command) > 1500:
            raise ValueError("Maximum items in 'Command' is 1500")
        entry_point = self.properties.get("EntryPoint")
        if entry_point and len(entry_point) > 1500:
            raise ValueError("Maximum items in 'EntryPoint' is 1500")
        working_directory = self.properties.get("WorkingDirectory")
        if working_directory and len(working_directory) > 1000:
            raise ValueError("Maximum length of 'WorkingDirectory' is 1000")


class VPCConfig(AWSProperty):

    props = {
        "SecurityGroupIds": (list, True),
        "SubnetIds": (list, True),
    }


class OnFailure(AWSProperty):
    props = {
        "Destination": (str, True),
    }


class OnSuccess(AWSProperty):
    props = {
        "Destination": (str, True),
    }


class DestinationConfig(AWSProperty):
    props = {
        "OnFailure": (OnFailure, False),
        "OnSuccess": (OnSuccess, False),
    }


class EventInvokeConfig(AWSObject):
    resource_type = "AWS::Lambda::EventInvokeConfig"

    props = {
        "DestinationConfig": (DestinationConfig, False),
        "FunctionName": (str, True),
        "MaximumEventAgeInSeconds": (integer, False),
        "MaximumRetryAttempts": (integer, False),
        "Qualifier": (str, True),
    }


class Endpoints(AWSProperty):
    props = {
        "KafkaBootstrapServers": ([str], False),
    }


class SelfManagedEventSource(AWSProperty):
    props = {
        "Endpoints": (Endpoints, False),
    }


class SourceAccessConfiguration(AWSProperty):
    props = {
        "Type": (str, False),
        "URI": (str, False),
    }


class EventSourceMapping(AWSObject):
    resource_type = "AWS::Lambda::EventSourceMapping"

    props = {
        "BatchSize": (integer, False),
        "BisectBatchOnFunctionError": (boolean, False),
        "DestinationConfig": (DestinationConfig, False),
        "Enabled": (boolean, False),
        "EventSourceArn": (str, False),
        "FunctionName": (str, True),
        "FunctionResponseTypes": ([str], False),
        "MaximumBatchingWindowInSeconds": (integer, False),
        "MaximumRecordAgeInSeconds": (integer, False),
        "MaximumRetryAttempts": (integer, False),
        "ParallelizationFactor": (integer, False),
        "PartialBatchResponse": (boolean, False),
        "Queues": ([str], False),
        "SelfManagedEventSource": (SelfManagedEventSource, False),
        "SourceAccessConfigurations": ([SourceAccessConfiguration], False),
        "StartingPosition": (str, False),
        "Topics": ([str], False),
        "TumblingWindowInSeconds": (integer, False),
    }


class DeadLetterConfig(AWSProperty):
    props = {
        "TargetArn": (str, False),
    }


class Environment(AWSProperty):
    props = {
        "Variables": (validate_variables_name, True),
    }


class FileSystemConfig(AWSProperty):
    props = {
        "Arn": (str, True),
        "LocalMountPath": (str, True),
    }


class TracingConfig(AWSProperty):
    props = {
        "Mode": (str, False),
    }


class Function(AWSObject):
    resource_type = "AWS::Lambda::Function"

    props = {
        "Code": (Code, True),
        "Description": (str, False),
        "DeadLetterConfig": (DeadLetterConfig, False),
        "Environment": (Environment, False),
        "FileSystemConfigs": ([FileSystemConfig], False),
        "FunctionName": (str, False),
        "Handler": (str, False),
        "ImageConfig": (ImageConfig, False),
        "KmsKeyArn": (str, False),
        "MemorySize": (validate_memory_size, False),
        "Layers": ([str], False),
        "PackageType": (validate_package_type, False),
        "ReservedConcurrentExecutions": (positive_integer, False),
        "Role": (str, True),
        "Runtime": (str, False),
        "Tags": (Tags, False),
        "Timeout": (positive_integer, False),
        "TracingConfig": (TracingConfig, False),
        "VpcConfig": (VPCConfig, False),
    }


class Permission(AWSObject):
    resource_type = "AWS::Lambda::Permission"

    props = {
        "Action": (str, True),
        "EventSourceToken": (str, False),
        "FunctionName": (str, True),
        "Principal": (str, True),
        "SourceAccount": (str, False),
        "SourceArn": (str, False),
    }


class VersionWeight(AWSProperty):

    props = {
        "FunctionVersion": (str, True),
        "FunctionWeight": (float, True),
    }


class AliasRoutingConfiguration(AWSProperty):

    props = {
        "AdditionalVersionWeights": ([VersionWeight], True),
    }


class ProvisionedConcurrencyConfiguration(AWSProperty):

    props = {
        "ProvisionedConcurrentExecutions": (integer, True),
    }


class Alias(AWSObject):
    resource_type = "AWS::Lambda::Alias"

    props = {
        "Description": (str, False),
        "FunctionName": (str, True),
        "FunctionVersion": (str, True),
        "Name": (str, True),
        "ProvisionedConcurrencyConfig": (ProvisionedConcurrencyConfiguration, False),
        "RoutingConfig": (AliasRoutingConfiguration, False),
    }


class AllowedPublishers(AWSProperty):
    props = {
        "SigningProfileVersionArns": ([str], True),
    }


class CodeSigningPolicies(AWSProperty):
    props = {
        "UntrustedArtifactOnDeployment": (str, True),
    }


class CodeSigningConfig(AWSObject):
    resource_type = "AWS::Lambda::CodeSigningConfig"

    props = {
        "AllowedPublishers": (AllowedPublishers, True),
        "CodeSigningPolicies": (CodeSigningPolicies, False),
        "Description": (str, False),
    }


class Version(AWSObject):
    resource_type = "AWS::Lambda::Version"

    props = {
        "CodeSha256": (str, False),
        "Description": (str, False),
        "FunctionName": (str, True),
        "ProvisionedConcurrencyConfig": (ProvisionedConcurrencyConfiguration, False),
    }


class Content(AWSProperty):
    props = {
        "S3Bucket": (str, True),
        "S3Key": (str, True),
        "S3ObjectVersion": (str, False),
    }


class LayerVersion(AWSObject):
    resource_type = "AWS::Lambda::LayerVersion"

    props = {
        "CompatibleRuntimes": ([str], False),
        "Content": (Content, True),
        "Description": (str, False),
        "LayerName": (str, False),
        "LicenseInfo": (str, False),
    }


class LayerVersionPermission(AWSObject):
    resource_type = "AWS::Lambda::LayerVersionPermission"

    props = {
        "Action": (str, True),
        "LayerVersionArn": (str, True),
        "OrganizationId": (str, False),
        "Principal": (str, True),
    }



#!/usr/bin/env python
"""
Converted from ElastiCache_Redis.template located at:
http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

In addition to troposphere, this script requires awacs (Amazon Web Access
Control Subsystem)
"""


import awacs
from awacs.aws import Allow, PolicyDocument, Principal, Statement
from awacs.sts import AssumeRole

import troposphere.ec2 as ec2
import troposphere.elasticache as elasticache
import troposphere.iam as iam
from troposphere import (
    Base64,
    FindInMap,
    GetAtt,
    Join,
    Output,
    Parameter,
    Ref,
    Tags,
    Template,
    cloudformation,
)
from troposphere.policies import CreationPolicy, ResourceSignal


def main():
    """
    Create a ElastiCache Redis Node and EC2 Instance
    """

    template = Template()

    # Description
    template.set_description(
        "AWS CloudFormation Sample Template ElastiCache_Redis:"
        "Sample template showing how to create an Amazon"
        "ElastiCache Redis Cluster. **WARNING** This template"
        "creates an Amazon EC2 Instance and an Amazon ElastiCache"
        "Cluster. You will be billed for the AWS resources used"
        "if you create a stack from this template."
    )

    # Mappings
    template.add_mapping(
        "AWSInstanceType2Arch",
        {
            "t1.micro": {"Arch": "PV64"},
            "t2.micro": {"Arch": "HVM64"},
            "t2.small": {"Arch": "HVM64"},
            "t2.medium": {"Arch": "HVM64"},
            "m1.small": {"Arch": "PV64"},
            "m1.medium": {"Arch": "PV64"},
            "m1.large": {"Arch": "PV64"},
            "m1.xlarge": {"Arch": "PV64"},
            "m2.xlarge": {"Arch": "PV64"},
            "m2.2xlarge": {"Arch": "PV64"},
            "m2.4xlarge": {"Arch": "PV64"},
            "m3.medium": {"Arch": "HVM64"},
            "m3.large": {"Arch": "HVM64"},
            "m3.xlarge": {"Arch": "HVM64"},
            "m3.2xlarge": {"Arch": "HVM64"},
            "c1.medium": {"Arch": "PV64"},
            "c1.xlarge": {"Arch": "PV64"},
            "c3.large": {"Arch": "HVM64"},
            "c3.xlarge": {"Arch": "HVM64"},
            "c3.2xlarge": {"Arch": "HVM64"},
            "c3.4xlarge": {"Arch": "HVM64"},
            "c3.8xlarge": {"Arch": "HVM64"},
            "c4.large": {"Arch": "HVM64"},
            "c4.xlarge": {"Arch": "HVM64"},
            "c4.2xlarge": {"Arch": "HVM64"},
            "c4.4xlarge": {"Arch": "HVM64"},
            "c4.8xlarge": {"Arch": "HVM64"},
            "g2.2xlarge": {"Arch": "HVMG2"},
            "r3.large": {"Arch": "HVM64"},
            "r3.xlarge": {"Arch": "HVM64"},
            "r3.2xlarge": {"Arch": "HVM64"},
            "r3.4xlarge": {"Arch": "HVM64"},
            "r3.8xlarge": {"Arch": "HVM64"},
            "i2.xlarge": {"Arch": "HVM64"},
            "i2.2xlarge": {"Arch": "HVM64"},
            "i2.4xlarge": {"Arch": "HVM64"},
            "i2.8xlarge": {"Arch": "HVM64"},
            "d2.xlarge": {"Arch": "HVM64"},
            "d2.2xlarge": {"Arch": "HVM64"},
            "d2.4xlarge": {"Arch": "HVM64"},
            "d2.8xlarge": {"Arch": "HVM64"},
            "hi1.4xlarge": {"Arch": "HVM64"},
            "hs1.8xlarge": {"Arch": "HVM64"},
            "cr1.8xlarge": {"Arch": "HVM64"},
            "cc2.8xlarge": {"Arch": "HVM64"},
        },
    )

    template.add_mapping(
        "AWSRegionArch2AMI",
        {
            "us-east-1": {
                "PV64": "ami-0f4cfd64",
                "HVM64": "ami-0d4cfd66",
                "HVMG2": "ami-5b05ba30",
            },
            "us-west-2": {
                "PV64": "ami-d3c5d1e3",
                "HVM64": "ami-d5c5d1e5",
                "HVMG2": "ami-a9d6c099",
            },
            "us-west-1": {
                "PV64": "ami-85ea13c1",
                "HVM64": "ami-87ea13c3",
                "HVMG2": "ami-37827a73",
            },
            "eu-west-1": {
                "PV64": "ami-d6d18ea1",
                "HVM64": "ami-e4d18e93",
                "HVMG2": "ami-72a9f105",
            },
            "eu-central-1": {
                "PV64": "ami-a4b0b7b9",
                "HVM64": "ami-a6b0b7bb",
                "HVMG2": "ami-a6c9cfbb",
            },
            "ap-northeast-1": {
                "PV64": "ami-1a1b9f1a",
                "HVM64": "ami-1c1b9f1c",
                "HVMG2": "ami-f644c4f6",
            },
            "ap-southeast-1": {
                "PV64": "ami-d24b4280",
                "HVM64": "ami-d44b4286",
                "HVMG2": "ami-12b5bc40",
            },
            "ap-southeast-2": {
                "PV64": "ami-ef7b39d5",
                "HVM64": "ami-db7b39e1",
                "HVMG2": "ami-b3337e89",
            },
            "sa-east-1": {
                "PV64": "ami-5b098146",
                "HVM64": "ami-55098148",
                "HVMG2": "NOT_SUPPORTED",
            },
            "cn-north-1": {
                "PV64": "ami-bec45887",
                "HVM64": "ami-bcc45885",
                "HVMG2": "NOT_SUPPORTED",
            },
        },
    )

    template.add_mapping(
        "Region2Principal",
        {
            "us-east-1": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "us-west-2": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "us-west-1": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "eu-west-1": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "ap-southeast-1": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "ap-northeast-1": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "ap-southeast-2": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "sa-east-1": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
            "cn-north-1": {
                "EC2Principal": "ec2.amazonaws.com.cn",
                "OpsWorksPrincipal": "opsworks.amazonaws.com.cn",
            },
            "eu-central-1": {
                "EC2Principal": "ec2.amazonaws.com",
                "OpsWorksPrincipal": "opsworks.amazonaws.com",
            },
        },
    )

    # Parameters
    cachenodetype = template.add_parameter(
        Parameter(
            "ClusterNodeType",
            Description="The compute and memory capacity of the nodes in the Redis"
            " Cluster",
            Type="String",
            Default="cache.m1.small",
            AllowedValues=[
                "cache.m1.small",
                "cache.m1.large",
                "cache.m1.xlarge",
                "cache.m2.xlarge",
                "cache.m2.2xlarge",
                "cache.m2.4xlarge",
                "cache.c1.xlarge",
            ],
            ConstraintDescription="must select a valid Cache Node type.",
        )
    )

    instancetype = template.add_parameter(
        Parameter(
            "InstanceType",
            Description="WebServer EC2 instance type",
            Type="String",
            Default="t2.micro",
            AllowedValues=[
                "t1.micro",
                "t2.micro",
                "t2.small",
                "t2.medium",
                "m1.small",
                "m1.medium",
                "m1.large",
                "m1.xlarge",
                "m2.xlarge",
                "m2.2xlarge",
                "m2.4xlarge",
                "m3.medium",
                "m3.large",
                "m3.xlarge",
                "m3.2xlarge",
                "c1.medium",
                "c1.xlarge",
                "c3.large",
                "c3.xlarge",
                "c3.2xlarge",
                "c3.4xlarge",
                "c3.8xlarge",
                "c4.large",
                "c4.xlarge",
                "c4.2xlarge",
                "c4.4xlarge",
                "c4.8xlarge",
                "g2.2xlarge",
                "r3.large",
                "r3.xlarge",
                "r3.2xlarge",
                "r3.4xlarge",
                "r3.8xlarge",
                "i2.xlarge",
                "i2.2xlarge",
                "i2.4xlarge",
                "i2.8xlarge",
                "d2.xlarge",
                "d2.2xlarge",
                "d2.4xlarge",
                "d2.8xlarge",
                "hi1.4xlarge",
                "hs1.8xlarge",
                "cr1.8xlarge",
                "cc2.8xlarge",
                "cg1.4xlarge",
            ],
            ConstraintDescription="must be a valid EC2 instance type.",
        )
    )

    keyname = template.add_parameter(
        Parameter(
            "KeyName",
            Description="Name of an existing EC2 KeyPair to enable SSH access"
            " to the instance",
            Type="AWS::EC2::KeyPair::KeyName",
            ConstraintDescription="must be the name of an existing EC2 KeyPair.",
        )
    )

    sshlocation = template.add_parameter(
        Parameter(
            "SSHLocation",
            Description="The IP address range that can be used to SSH to"
            " the EC2 instances",
            Type="String",
            MinLength="9",
            MaxLength="18",
            Default="0.0.0.0/0",
            AllowedPattern="(\\d{1,3})\\.(\\d{1,3})\\."
            "(\\d{1,3})\\.(\\d{1,3})/(\\d{1,2})",
            ConstraintDescription="must be a valid IP CIDR range of the"
            " form x.x.x.x/x.",
        )
    )

    # Resources
    webserverrole = template.add_resource(
        iam.Role(
            "WebServerRole",
            AssumeRolePolicyDocument=PolicyDocument(
                Statement=[
                    Statement(
                        Effect=Allow,
                        Action=[AssumeRole],
                        Principal=Principal(
                            "Service",
                            [
                                FindInMap(
                                    "Region2Principal",
                                    Ref("AWS::Region"),
                                    "EC2Principal",
                                )
                            ],
                        ),
                    )
                ]
            ),
            Path="/",
        )
    )

    template.add_resource(
        iam.PolicyType(
            "WebServerRolePolicy",
            PolicyName="WebServerRole",
            PolicyDocument=PolicyDocument(
                Statement=[
                    awacs.aws.Statement(
                        Action=[
                            awacs.aws.Action("elasticache", "DescribeCacheClusters")
                        ],
                        Resource=["*"],
                        Effect=awacs.aws.Allow,
                    )
                ]
            ),
            Roles=[Ref(webserverrole)],
        )
    )

    webserverinstanceprofile = template.add_resource(
        iam.InstanceProfile(
            "WebServerInstanceProfile",
            Path="/",
            Roles=[Ref(webserverrole)],
        )
    )

    webserversg = template.add_resource(
        ec2.SecurityGroup(
            "WebServerSecurityGroup",
            GroupDescription="Enable HTTP and SSH access",
            SecurityGroupIngress=[
                ec2.SecurityGroupRule(
                    IpProtocol="tcp",
                    FromPort="22",
                    ToPort="22",
                    CidrIp=Ref(sshlocation),
                ),
                ec2.SecurityGroupRule(
                    IpProtocol="tcp",
                    FromPort="80",
                    ToPort="80",
                    CidrIp="0.0.0.0/0",
                ),
            ],
        )
    )

    webserverinstance = template.add_resource(
        ec2.Instance(
            "WebServerInstance",
            Metadata=cloudformation.Metadata(
                cloudformation.Init(
                    {
                        "config": cloudformation.InitConfig(
                            packages={
                                "yum": {
                                    "httpd": [],
                                    "php": [],
                                    "php-devel": [],
                                    "gcc": [],
                                    "make": [],
                                }
                            },
                            files=cloudformation.InitFiles(
                                {
                                    "/var/www/html/index.php": cloudformation.InitFile(
                                        content=Join(
                                            "",
                                            [
                                                "<?php\n",
                                                'echo "<h1>AWS CloudFormation sample'
                                                " application for Amazon ElastiCache"
                                                ' Redis Cluster</h1>";\n',
                                                "\n",
                                                "$cluster_config = json_decode("
                                                "file_get_contents('/tmp/cacheclusterconfig'"
                                                "), true);\n",
                                                "$endpoint = $cluster_config['CacheClusters"
                                                "'][0]['CacheNodes'][0]['Endpoint']['Add"
                                                "ress'];\n",
                                                "$port = $cluster_config['CacheClusters'][0]"
                                                "['CacheNodes'][0]['Endpoint']['Port'];"
                                                "\n",
                                                "\n",
                                                'echo "<p>Connecting to Redis Cache Cluster '
                                                "node '{$endpoint}' on port {$port}</p>\";"
                                                "\n",
                                                "\n",
                                                "$redis=new Redis();\n",
                                                "$redis->connect($endpoint, $port);\n",
                                                "$redis->set('testkey', 'Hello World!');"
                                                "\n",
                                                "$return = $redis->get('testkey');\n",
                                                "\n",
                                                'echo "<p>Retrieved value: $return</p>";'
                                                "\n",
                                                "?>\n",
                                            ],
                                        ),
                                        mode="000644",
                                        owner="apache",
                                        group="apache",
                                    ),
                                    "/etc/cron.d/get_cluster_config": cloudformation.InitFile(
                                        content="*/5 * * * * root"
                                        " /usr/local/bin/get_cluster_config",
                                        mode="000644",
                                        owner="root",
                                        group="root",
                                    ),
                                    "/usr/local/bin/get_cluster_config": cloudformation.InitFile(
                                        content=Join(
                                            "",
                                            [
                                                "#! /bin/bash\n",
                                                "aws elasticache describe-cache-clusters ",
                                                "         --cache-cluster-id ",
                                                Ref("RedisCluster"),
                                                "         --show-cache-node-info"
                                                " --region ",
                                                Ref("AWS::Region"),
                                                " > /tmp/cacheclusterconfig\n",
                                            ],
                                        ),
                                        mode="000755",
                                        owner="root",
                                        group="root",
                                    ),
                                    "/usr/local/bin/install_phpredis": cloudformation.InitFile(
                                        content=Join(
                                            "",
                                            [
                                                "#! /bin/bash\n",
                                                "cd /tmp\n",
                                                "wget https://github.com/nicolasff/"
                                                "phpredis/zipball/master -O phpredis.zip"
                                                "\n",
                                                "unzip phpredis.zip\n",
                                                "cd nicolasff-phpredis-*\n",
                                                "phpize\n",
                                                "./configure\n",
                                                "make && make install\n",
                                                "touch /etc/php.d/redis.ini\n",
                                                "echo extension=redis.so > /etc/php.d/"
                                                "redis.ini\n",
                                            ],
                                        ),
                                        mode="000755",
                                        owner="root",
                                        group="root",
                                    ),
                                    "/etc/cfn/cfn-hup.conf": cloudformation.InitFile(
                                        content=Join(
                                            "",
                                            [
                                                "[main]\n",
                                                "stack=",
                                                Ref("AWS::StackId"),
                                                "\n",
                                                "region=",
                                                Ref("AWS::Region"),
                                                "\n",
                                            ],
                                        ),
                                        mode="000400",
                                        owner="root",
                                        group="root",
                                    ),
                                    "/etc/cfn/hooks.d/cfn-auto-reloader.conf": cloudformation.InitFile(
                                        content=Join(
                                            "",
                                            [
                                                "[cfn-auto-reloader-hook]\n",
                                                "triggers=post.update\n",
                                                "path=Resources.WebServerInstance.Metadata"
                                                ".AWS::CloudFormation::Init\n",
                                                "action=/opt/aws/bin/cfn-init -v ",
                                                "         --stack ",
                                                Ref("AWS::StackName"),
                                                "         --resource WebServerInstance ",
                                                "         --region ",
                                                Ref("AWS::Region"),
                                                "\n",
                                                "runas=root\n",
                                            ],
                                        ),
                                        # Why doesn't the Amazon template have this?
                                        # mode='000400',
                                        # owner='root',
                                        # group='root'
                                    ),
                                }
                            ),
                            commands={
                                "01-install_phpredis": {
                                    "command": "/usr/local/bin/install_phpredis"
                                },
                                "02-get-cluster-config": {
                                    "command": "/usr/local/bin/get_cluster_config"
                                },
                            },
                            services={
                                "sysvinit": cloudformation.InitServices(
                                    {
                                        "httpd": cloudformation.InitService(
                                            enabled=True,
                                            ensureRunning=True,
                                        ),
                                        "cfn-hup": cloudformation.InitService(
                                            enabled=True,
                                            ensureRunning=True,
                                            files=[
                                                "/etc/cfn/cfn-hup.conf",
                                                "/etc/cfn/hooks.d/"
                                                "cfn-auto-reloader.conf",
                                            ],
                                        ),
                                    }
                                ),
                            },
                        )
                    }
                )
            ),
            ImageId=FindInMap(
                "AWSRegionArch2AMI",
                Ref("AWS::Region"),
                FindInMap("AWSInstanceType2Arch", Ref(instancetype), "Arch"),
            ),
            InstanceType=Ref(instancetype),
            SecurityGroups=[Ref(webserversg)],
            KeyName=Ref(keyname),
            IamInstanceProfile=Ref(webserverinstanceprofile),
            UserData=Base64(
                Join(
                    "",
                    [
                        "#!/bin/bash -xe\n",
                        "yum update -y aws-cfn-bootstrap\n",
                        "# Setup the PHP sample application\n",
                        "/opt/aws/bin/cfn-init -v ",
                        "         --stack ",
                        Ref("AWS::StackName"),
                        "         --resource WebServerInstance ",
                        "         --region ",
                        Ref("AWS::Region"),
                        "\n",
                        "# Signal the status of cfn-init\n",
                        "/opt/aws/bin/cfn-signal -e $? ",
                        "         --stack ",
                        Ref("AWS::StackName"),
                        "         --resource WebServerInstance ",
                        "         --region ",
                        Ref("AWS::Region"),
                        "\n",
                    ],
                )
            ),
            CreationPolicy=CreationPolicy(
                ResourceSignal=ResourceSignal(Timeout="PT15M")
            ),
            Tags=Tags(
                Application=Ref("AWS::StackId"), Details="Created using Troposhpere"
            ),
        )
    )

    redisclustersg = template.add_resource(
        elasticache.SecurityGroup(
            "RedisClusterSecurityGroup",
            Description="Lock the cluster down",
        )
    )

    template.add_resource(
        elasticache.SecurityGroupIngress(
            "RedisClusterSecurityGroupIngress",
            CacheSecurityGroupName=Ref(redisclustersg),
            EC2SecurityGroupName=Ref(webserversg),
        )
    )

    template.add_resource(
        elasticache.CacheCluster(
            "RedisCluster",
            Engine="redis",
            CacheNodeType=Ref(cachenodetype),
            NumCacheNodes="1",
            CacheSecurityGroupNames=[Ref(redisclustersg)],
        )
    )

    # Outputs
    template.add_output(
        [
            Output(
                "WebsiteURL",
                Description="Application URL",
                Value=Join(
                    "",
                    [
                        "http://",
                        GetAtt(webserverinstance, "PublicDnsName"),
                    ],
                ),
            )
        ]
    )

    # Print CloudFormation Template
    print(template.to_json())


if __name__ == "__main__":
    main()



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 3.3.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean


class DeliveryOptions(AWSProperty):
    props = {
        "SendingPoolName": (str, False),
    }


class ReputationOptions(AWSProperty):
    props = {
        "ReputationMetricsEnabled": (boolean, False),
    }


class SendingOptions(AWSProperty):
    props = {
        "SendingEnabled": (boolean, False),
    }


class TrackingOptions(AWSProperty):
    props = {
        "CustomRedirectDomain": (str, False),
    }


class ConfigurationSet(AWSObject):
    resource_type = "AWS::PinpointEmail::ConfigurationSet"

    props = {
        "DeliveryOptions": (DeliveryOptions, False),
        "Name": (str, True),
        "ReputationOptions": (ReputationOptions, False),
        "SendingOptions": (SendingOptions, False),
        "Tags": (Tags, False),
        "TrackingOptions": (TrackingOptions, False),
    }


class DimensionConfiguration(AWSProperty):
    props = {
        "DefaultDimensionValue": (str, True),
        "DimensionName": (str, True),
        "DimensionValueSource": (str, True),
    }


class CloudWatchDestination(AWSProperty):
    props = {
        "DimensionConfigurations": ([DimensionConfiguration], False),
    }


class KinesisFirehoseDestination(AWSProperty):
    props = {
        "DeliveryStreamArn": (str, True),
        "IamRoleArn": (str, True),
    }


class PinpointDestination(AWSProperty):
    props = {
        "ApplicationArn": (str, False),
    }


class SnsDestination(AWSProperty):
    props = {
        "TopicArn": (str, True),
    }


class EventDestination(AWSProperty):
    props = {
        "CloudWatchDestination": (CloudWatchDestination, False),
        "Enabled": (boolean, False),
        "KinesisFirehoseDestination": (KinesisFirehoseDestination, False),
        "MatchingEventTypes": ([str], True),
        "PinpointDestination": (PinpointDestination, False),
        "SnsDestination": (SnsDestination, False),
    }


class ConfigurationSetEventDestination(AWSObject):
    resource_type = "AWS::PinpointEmail::ConfigurationSetEventDestination"

    props = {
        "ConfigurationSetName": (str, True),
        "EventDestination": (EventDestination, False),
        "EventDestinationName": (str, True),
    }


class DedicatedIpPool(AWSObject):
    resource_type = "AWS::PinpointEmail::DedicatedIpPool"

    props = {
        "PoolName": (str, False),
        "Tags": (Tags, False),
    }


class MailFromAttributes(AWSProperty):
    props = {
        "BehaviorOnMxFailure": (str, False),
        "MailFromDomain": (str, False),
    }


class Identity(AWSObject):
    resource_type = "AWS::PinpointEmail::Identity"

    props = {
        "DkimSigningEnabled": (boolean, False),
        "FeedbackForwardingEnabled": (boolean, False),
        "MailFromAttributes": (MailFromAttributes, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (
    boolean,
    cloudfront_cache_cookie_behavior,
    cloudfront_cache_header_behavior,
    cloudfront_cache_query_string_behavior,
    cloudfront_event_type,
    cloudfront_forward_type,
    cloudfront_origin_request_cookie_behavior,
    cloudfront_origin_request_header_behavior,
    cloudfront_origin_request_query_string_behavior,
    cloudfront_restriction_type,
    cloudfront_viewer_protocol_policy,
    integer,
    network_port,
    positive_integer,
    priceclass_type,
)


class Cookies(AWSProperty):
    props = {
        "Forward": (cloudfront_forward_type, True),
        "WhitelistedNames": ([str], False),
    }


class ForwardedValues(AWSProperty):
    props = {
        "Cookies": (Cookies, False),
        "Headers": ([str], False),
        "QueryString": (boolean, True),
        "QueryStringCacheKeys": ([str], False),
    }


class FunctionAssociation(AWSProperty):
    props = {
        "EventType": (str, False),
        "FunctionARN": (str, False),
    }


class LambdaFunctionAssociation(AWSProperty):
    props = {
        "EventType": (cloudfront_event_type, False),
        "IncludeBody": (boolean, False),
        "LambdaFunctionARN": (str, False),
    }


class CacheBehavior(AWSProperty):
    props = {
        "AllowedMethods": ([str], False),
        "CachePolicyId": (str, False),
        "CachedMethods": ([str], False),
        "Compress": (boolean, False),
        "DefaultTTL": (integer, False),
        "FieldLevelEncryptionId": (str, False),
        "ForwardedValues": (ForwardedValues, False),
        "FunctionAssociations": ([FunctionAssociation], False),
        "LambdaFunctionAssociations": ([LambdaFunctionAssociation], False),
        "MaxTTL": (integer, False),
        "MinTTL": (integer, False),
        "OriginRequestPolicyId": (str, False),
        "PathPattern": (str, True),
        "RealtimeLogConfigArn": (str, False),
        "SmoothStreaming": (boolean, False),
        "TargetOriginId": (str, True),
        "TrustedKeyGroups": ([str], False),
        "TrustedSigners": ([str], False),
        "ViewerProtocolPolicy": (cloudfront_viewer_protocol_policy, True),
    }


class DefaultCacheBehavior(AWSProperty):
    props = {
        "AllowedMethods": ([str], False),
        "CachePolicyId": (str, False),
        "CachedMethods": ([str], False),
        "Compress": (boolean, False),
        "DefaultTTL": (integer, False),
        "FieldLevelEncryptionId": (str, False),
        "ForwardedValues": (ForwardedValues, False),
        "FunctionAssociations": ([FunctionAssociation], False),
        "LambdaFunctionAssociations": ([LambdaFunctionAssociation], False),
        "MaxTTL": (integer, False),
        "MinTTL": (integer, False),
        "OriginRequestPolicyId": (str, False),
        "RealtimeLogConfigArn": (str, False),
        "SmoothStreaming": (boolean, False),
        "TargetOriginId": (str, True),
        "TrustedKeyGroups": ([str], False),
        "TrustedSigners": (list, False),
        "ViewerProtocolPolicy": (cloudfront_viewer_protocol_policy, True),
    }


class S3Origin(AWSProperty):
    props = {
        "DomainName": (str, True),
        "OriginAccessIdentity": (str, False),
    }


class CustomOriginConfig(AWSProperty):
    props = {
        "HTTPPort": (network_port, False),
        "HTTPSPort": (network_port, False),
        "OriginKeepaliveTimeout": (integer, False),
        "OriginProtocolPolicy": (str, True),
        "OriginReadTimeout": (integer, False),
        "OriginSSLProtocols": ([str], False),
    }


class OriginCustomHeader(AWSProperty):
    props = {
        "HeaderName": (str, True),
        "HeaderValue": (str, True),
    }


class S3OriginConfig(AWSProperty):
    props = {
        "OriginAccessIdentity": (str, False),
    }


class OriginShield(AWSProperty):
    props = {
        "Enabled": (boolean, True),
        "OriginShieldRegion": (str, False),
    }


class Origin(AWSProperty):
    props = {
        "ConnectionAttempts": (integer, False),
        "ConnectionTimeout": (integer, False),
        "CustomOriginConfig": (CustomOriginConfig, False),
        "DomainName": (str, True),
        "Id": (str, True),
        "OriginCustomHeaders": ([OriginCustomHeader], False),
        "OriginPath": (str, False),
        "OriginShield": (OriginShield, False),
        "S3OriginConfig": (S3OriginConfig, False),
    }


class Logging(AWSProperty):
    props = {
        "Bucket": (str, True),
        "IncludeCookies": (boolean, False),
        "Prefix": (str, False),
    }


class CustomErrorResponse(AWSProperty):
    props = {
        "ErrorCachingMinTTL": (positive_integer, False),
        "ErrorCode": (positive_integer, True),
        "ResponseCode": (positive_integer, False),
        "ResponsePagePath": (str, False),
    }


class GeoRestriction(AWSProperty):
    props = {
        "Locations": ([str], False),
        "RestrictionType": (cloudfront_restriction_type, True),
    }


class Restrictions(AWSProperty):
    props = {
        "GeoRestriction": (GeoRestriction, True),
    }


class ViewerCertificate(AWSProperty):
    props = {
        "AcmCertificateArn": (str, False),
        "CloudFrontDefaultCertificate": (boolean, False),
        "IamCertificateId": (str, False),
        "MinimumProtocolVersion": (str, False),
        "SslSupportMethod": (str, False),
    }


class StatusCodes(AWSProperty):
    props = {
        "Items": ([integer], True),
        "Quantity": (integer, True),
    }


class OriginGroupFailoverCriteria(AWSProperty):
    props = {
        "StatusCodes": (StatusCodes, True),
    }


class OriginGroupMember(AWSProperty):
    props = {
        "OriginId": (str, True),
    }


class OriginGroupMembers(AWSProperty):
    props = {
        "Items": ([OriginGroupMember], False),
        "Quantity": (integer, False),
    }


class OriginGroup(AWSProperty):
    props = {
        "FailoverCriteria": (OriginGroupFailoverCriteria, True),
        "Id": (str, True),
        "Members": (OriginGroupMembers, True),
    }


class OriginGroups(AWSProperty):
    props = {
        "Items": ([OriginGroup], False),
        "Quantity": (integer, False),
    }


class DistributionConfig(AWSProperty):
    props = {
        "Aliases": (list, False),
        "CacheBehaviors": ([CacheBehavior], False),
        "Comment": (str, False),
        "CustomErrorResponses": ([CustomErrorResponse], False),
        "DefaultCacheBehavior": (DefaultCacheBehavior, True),
        "DefaultRootObject": (str, False),
        "Enabled": (boolean, True),
        "HttpVersion": (str, False),
        "IPV6Enabled": (boolean, False),
        "Logging": (Logging, False),
        "Origins": ([Origin], True),
        "OriginGroups": (OriginGroups, False),
        "PriceClass": (priceclass_type, False),
        "Restrictions": (Restrictions, False),
        "ViewerCertificate": (ViewerCertificate, False),
        "WebACLId": (str, False),
    }


class Distribution(AWSObject):
    resource_type = "AWS::CloudFront::Distribution"

    props = {
        "DistributionConfig": (DistributionConfig, True),
        "Tags": ((Tags, list), False),
    }


class CloudFrontOriginAccessIdentityConfig(AWSProperty):
    props = {
        "Comment": (str, True),
    }


class CloudFrontOriginAccessIdentity(AWSObject):
    resource_type = "AWS::CloudFront::CloudFrontOriginAccessIdentity"

    props = {
        "CloudFrontOriginAccessIdentityConfig": (
            CloudFrontOriginAccessIdentityConfig,
            True,
        ),
    }


class TrustedSigners(AWSProperty):
    props = {
        "AwsAccountNumbers": ([str], False),
        "Enabled": (boolean, True),
    }


class StreamingDistributionConfig(AWSProperty):
    props = {
        "Aliases": ([str], False),
        "Comment": (str, True),
        "Enabled": (boolean, True),
        "Logging": (Logging, False),
        "PriceClass": (priceclass_type, False),
        "S3Origin": (S3Origin, True),
        "TrustedSigners": (TrustedSigners, True),
    }


class StreamingDistribution(AWSObject):
    resource_type = "AWS::CloudFront::StreamingDistribution"

    props = {
        "StreamingDistributionConfig": (StreamingDistributionConfig, True),
        "Tags": ((Tags, list), False),
    }


class CacheCookiesConfig(AWSProperty):
    props = {
        "CookieBehavior": (cloudfront_cache_cookie_behavior, True),
        "Cookies": ([str], False),
    }


class CacheHeadersConfig(AWSProperty):
    props = {
        "HeaderBehavior": (cloudfront_cache_header_behavior, True),
        "Headers": ([str], False),
    }


class CacheQueryStringsConfig(AWSProperty):
    props = {
        "QueryStringBehavior": (cloudfront_cache_query_string_behavior, True),
        "QueryStrings": ([str], False),
    }


class ParametersInCacheKeyAndForwardedToOrigin(AWSProperty):
    props = {
        "CookiesConfig": (CacheCookiesConfig, True),
        "EnableAcceptEncodingGzip": (boolean, True),
        "EnableAcceptEncodingBrotli": (boolean, False),
        "HeadersConfig": (CacheHeadersConfig, True),
        "QueryStringsConfig": (CacheQueryStringsConfig, True),
    }


class CachePolicyConfig(AWSProperty):
    props = {
        "Comment": (str, False),
        "DefaultTTL": (integer, True),
        "MaxTTL": (integer, True),
        "MinTTL": (integer, True),
        "Name": (str, True),
        "ParametersInCacheKeyAndForwardedToOrigin": (
            ParametersInCacheKeyAndForwardedToOrigin,
            True,
        ),
    }


class CachePolicy(AWSObject):
    resource_type = "AWS::CloudFront::CachePolicy"

    props = {
        "CachePolicyConfig": (CachePolicyConfig, True),
    }


class OriginRequestCookiesConfig(AWSProperty):
    props = {
        "CookieBehavior": (cloudfront_origin_request_cookie_behavior, True),
        "Cookies": ([str], False),
    }


class OriginRequestHeadersConfig(AWSProperty):
    props = {
        "HeaderBehavior": (cloudfront_origin_request_header_behavior, True),
        "Headers": ([str], False),
    }


class OriginRequestQueryStringsConfig(AWSProperty):
    props = {
        "QueryStringBehavior": (cloudfront_origin_request_query_string_behavior, True),
        "QueryStrings": ([str], False),
    }


class OriginRequestPolicyConfig(AWSProperty):
    props = {
        "Comment": (str, False),
        "CookiesConfig": (OriginRequestCookiesConfig, True),
        "HeadersConfig": (OriginRequestHeadersConfig, True),
        "Name": (str, True),
        "QueryStringsConfig": (OriginRequestQueryStringsConfig, True),
    }


class OriginRequestPolicy(AWSObject):
    resource_type = "AWS::CloudFront::OriginRequestPolicy"

    props = {
        "OriginRequestPolicyConfig": (OriginRequestPolicyConfig, True),
    }


class FunctionConfig(AWSProperty):
    props = {
        "Comment": (str, True),
        "Runtime": (str, True),
    }


class FunctionMetadata(AWSProperty):
    props = {
        "FunctionARN": (str, False),
    }


class Function(AWSObject):
    resource_type = "AWS::CloudFront::Function"

    props = {
        "AutoPublish": (boolean, False),
        "FunctionCode": (str, False),
        "FunctionConfig": (FunctionConfig, False),
        "FunctionMetadata": (FunctionMetadata, False),
        "Name": (str, True),
    }


class KeyGroupConfig(AWSProperty):
    props = {
        "Comment": (str, False),
        "Items": ([str], True),
        "Name": (str, True),
    }


class KeyGroup(AWSObject):
    resource_type = "AWS::CloudFront::KeyGroup"

    props = {
        "KeyGroupConfig": (KeyGroupConfig, True),
    }


class PublicKeyConfig(AWSProperty):
    props = {
        "CallerReference": (str, True),
        "Comment": (str, False),
        "EncodedKey": (str, True),
        "Name": (str, True),
    }


class PublicKey(AWSObject):
    resource_type = "AWS::CloudFront::PublicKey"

    props = {
        "PublicKeyConfig": (PublicKeyConfig, True),
    }


class KinesisStreamConfig(AWSProperty):
    props = {
        "RoleArn": (str, True),
        "StreamArn": (str, True),
    }


class EndPoint(AWSProperty):
    props = {
        "KinesisStreamConfig": (KinesisStreamConfig, True),
        "StreamType": (str, True),
    }


class RealtimeLogConfig(AWSObject):
    resource_type = "AWS::CloudFront::RealtimeLogConfig"

    props = {
        "EndPoints": ([EndPoint], True),
        "Fields": ([str], True),
        "Name": (str, True),
        "SamplingRate": (float, True),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from . import AWSObject, AWSProperty
from .validators import boolean, integer


class Grant(AWSObject):
    resource_type = "AWS::LicenseManager::Grant"

    props = {
        "AllowedOperations": ([str], False),
        "GrantName": (str, False),
        "HomeRegion": (str, False),
        "LicenseArn": (str, False),
        "Principals": ([str], False),
        "Status": (str, False),
    }


class BorrowConfiguration(AWSProperty):
    props = {
        "AllowEarlyCheckIn": (boolean, True),
        "MaxTimeToLiveInMinutes": (integer, True),
    }


class ProvisionalConfiguration(AWSProperty):
    props = {
        "MaxTimeToLiveInMinutes": (integer, True),
    }


class ConsumptionConfiguration(AWSProperty):
    props = {
        "BorrowConfiguration": (BorrowConfiguration, False),
        "ProvisionalConfiguration": (ProvisionalConfiguration, False),
        "RenewType": (str, False),
    }


class Entitlement(AWSProperty):
    props = {
        "AllowCheckIn": (boolean, False),
        "MaxCount": (integer, False),
        "Name": (str, True),
        "Overage": (boolean, False),
        "Unit": (str, True),
        "Value": (str, False),
    }


class IssuerData(AWSProperty):
    props = {
        "Name": (str, True),
        "SignKey": (str, False),
    }


class Metadata(AWSProperty):
    props = {
        "Name": (str, True),
        "Value": (str, True),
    }


class ValidityDateFormat(AWSProperty):
    props = {
        "Begin": (str, True),
        "End": (str, True),
    }


class License(AWSObject):
    resource_type = "AWS::LicenseManager::License"

    props = {
        "Beneficiary": (str, False),
        "ConsumptionConfiguration": (ConsumptionConfiguration, True),
        "Entitlements": ([Entitlement], True),
        "HomeRegion": (str, True),
        "Issuer": (IssuerData, True),
        "LicenseMetadata": ([Metadata], False),
        "LicenseName": (str, True),
        "ProductName": (str, True),
        "ProductSKU": (str, False),
        "Status": (str, False),
        "Validity": (ValidityDateFormat, True),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from . import AWSObject, AWSProperty, Tags
from .compat import policytypes


class PublicRepository(AWSObject):
    resource_type = "AWS::ECR::PublicRepository"

    props = {
        "RepositoryCatalogData": (dict, False),
        "RepositoryName": (str, False),
        "RepositoryPolicyText": (policytypes, False),
        "Tags": (Tags, False),
    }


class RegistryPolicy(AWSObject):
    resource_type = "AWS::ECR::RegistryPolicy"

    props = {
        "PolicyText": (policytypes, True),
    }


class ReplicationDestination(AWSProperty):
    props = {
        "Region": (str, True),
        "RegistryId": (str, True),
    }


class ReplicationRule(AWSProperty):
    props = {
        "Destinations": ([ReplicationDestination], True),
    }


class ReplicationConfigurationProperty(AWSProperty):
    props = {
        "Rules": ([ReplicationRule], True),
    }


class ReplicationConfiguration(AWSObject):
    resource_type = "AWS::ECR::ReplicationConfiguration"

    props = {
        "ReplicationConfigurationProperty": (ReplicationConfigurationProperty, True),
    }


class EncryptionConfiguration(AWSProperty):
    props = {
        "EncryptionType": (str, True),
        "KmsKey": (str, False),
    }


class LifecyclePolicy(AWSProperty):
    props = {
        "LifecyclePolicyText": (str, False),
        "RegistryId": (str, False),
    }


class ImageScanningConfiguration(AWSProperty):
    props = {
        "ScanOnPush": (bool, False),
    }


class Repository(AWSObject):
    resource_type = "AWS::ECR::Repository"

    props = {
        "EncryptionConfiguration": (EncryptionConfiguration, False),
        "ImageScanningConfiguration": (ImageScanningConfiguration, False),
        "ImageTagMutability": (str, False),
        "LifecyclePolicy": (LifecyclePolicy, False),
        "RepositoryName": (str, False),
        "RepositoryPolicyText": (policytypes, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer

WebServer = "WebServer"
Worker = "Worker"
WebServerType = "Standard"
WorkerType = "SQS/HTTP"


class MaxAgeRule(AWSProperty):
    props = {
        "DeleteSourceFromS3": (boolean, False),
        "Enabled": (boolean, False),
        "MaxAgeInDays": (integer, False),
    }


class MaxCountRule(AWSProperty):
    props = {
        "DeleteSourceFromS3": (boolean, False),
        "Enabled": (boolean, False),
        "MaxCount": (integer, False),
    }


class ApplicationVersionLifecycleConfig(AWSProperty):
    props = {
        "MaxAgeRule": (MaxAgeRule, False),
        "MaxCountRule": (MaxCountRule, False),
    }


class SourceBundle(AWSProperty):
    props = {
        "S3Bucket": (str, True),
        "S3Key": (str, True),
    }


class SourceConfiguration(AWSProperty):
    props = {
        "ApplicationName": (str, True),
        "TemplateName": (str, True),
    }


class ApplicationResourceLifecycleConfig(AWSProperty):
    props = {
        "ServiceRole": (str, False),
        "VersionLifecycleConfig": (ApplicationVersionLifecycleConfig, False),
    }


class OptionSetting(AWSProperty):
    props = {
        "Namespace": (str, True),
        "OptionName": (str, True),
        "ResourceName": (str, False),
        "Value": (str, False),
    }


class Application(AWSObject):
    resource_type = "AWS::ElasticBeanstalk::Application"

    props = {
        "ApplicationName": (str, False),
        "Description": (str, False),
        "ResourceLifecycleConfig": (ApplicationResourceLifecycleConfig, False),
    }


class ApplicationVersion(AWSObject):
    resource_type = "AWS::ElasticBeanstalk::ApplicationVersion"

    props = {
        "ApplicationName": (str, True),
        "Description": (str, False),
        "SourceBundle": (SourceBundle, False),
    }


class ConfigurationTemplate(AWSObject):
    resource_type = "AWS::ElasticBeanstalk::ConfigurationTemplate"

    props = {
        "ApplicationName": (str, True),
        "Description": (str, False),
        "EnvironmentId": (str, False),
        "OptionSettings": ([OptionSetting], False),
        "PlatformArn": (str, False),
        "SolutionStackName": (str, False),
        "SourceConfiguration": (SourceConfiguration, False),
    }


def validate_tier_name(name):
    valid_names = [WebServer, Worker]
    if name not in valid_names:
        raise ValueError("Tier name needs to be one of %r" % valid_names)
    return name


def validate_tier_type(tier_type):
    valid_types = [WebServerType, WorkerType]
    if tier_type not in valid_types:
        raise ValueError("Tier type needs to be one of %r" % valid_types)
    return tier_type


class Tier(AWSProperty):
    props = {
        "Name": (validate_tier_name, False),
        "Type": (validate_tier_type, False),
        "Version": (str, False),
    }


class Environment(AWSObject):
    resource_type = "AWS::ElasticBeanstalk::Environment"

    props = {
        "ApplicationName": (str, True),
        "CNAMEPrefix": (str, False),
        "Description": (str, False),
        "EnvironmentName": (str, False),
        "OperationsRole": (str, False),
        "OptionSettings": ([OptionSetting], False),
        "PlatformArn": (str, False),
        "SolutionStackName": (str, False),
        "Tags": (Tags, False),
        "TemplateName": (str, False),
        "Tier": (Tier, False),
        "VersionLabel": (str, False),
    }



# Copyright (c) 2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer

VALID_SIGNIN_ALGORITHM = (
    "SHA256WITHECDSA",
    "SHA256WITHRSA",
    "SHA384WITHECDSA",
    "SHA384WITHRSA",
    "SHA512WITHECDSA",
    "SHA512WITHRSA",
)
VALID_VALIDITY_TYPE = ("ABSOLUTE", "DAYS", "END_DATE", "MONTHS", "YEARS")
VALID_KEY_ALGORITHM = ("EC_prime256v1", "EC_secp384r1", "RSA_2048", "RSA_4096")
VALID_CERTIFICATEAUTHORITY_TYPE = ("ROOT", "SUBORDINATE")


def validate_validity_type(validity_type):
    """Certificate Validity Type validation rule."""
    if validity_type not in VALID_VALIDITY_TYPE:
        raise ValueError(
            "Certificate Validity Type must be one of: %s"
            % ", ".join(VALID_VALIDITY_TYPE)
        )
    return validity_type


def validate_signing_algorithm(signing_algorithm):
    """Certificate SigningAlgorithm validation rule."""
    if signing_algorithm not in VALID_SIGNIN_ALGORITHM:
        raise ValueError(
            "Certificate SigningAlgorithm must be one of: %s"
            % ", ".join(VALID_SIGNIN_ALGORITHM)
        )
    return signing_algorithm


def validate_key_algorithm(key_algorithm):
    """CertificateAuthority KeyAlgorithm validation rule."""
    if key_algorithm not in VALID_KEY_ALGORITHM:
        raise ValueError(
            "CertificateAuthority KeyAlgorithm must be one of: %s"
            % ", ".join(VALID_KEY_ALGORITHM)  # NOQA
        )
    return key_algorithm


def validate_certificateauthority_type(certificateauthority_type):
    """CertificateAuthority Type validation rule."""
    if certificateauthority_type not in VALID_CERTIFICATEAUTHORITY_TYPE:
        raise ValueError(
            "CertificateAuthority Type must be one of: %s"
            % ", ".join(VALID_CERTIFICATEAUTHORITY_TYPE)
        )
    return certificateauthority_type


class Qualifier(AWSProperty):
    props = {
        "CpsUri": (str, True),
    }


class PolicyQualifierInfo(AWSProperty):
    props = {
        "PolicyQualifierId": (str, True),
        "Qualifier": (Qualifier, True),
    }


class PolicyQualifierInfoList(AWSProperty):
    props = {
        "PolicyQualifierInfoList": ([PolicyQualifierInfo], False),
    }


class PolicyInformation(AWSProperty):
    props = {
        "CertPolicyId": (str, True),
        "PolicyQualifiers": (PolicyQualifierInfoList, False),
    }


class CertificatePolicyList(AWSProperty):
    props = {
        "CertificatePolicyList": ([PolicyInformation], False),
    }


class ExtendedKeyUsage(AWSProperty):
    props = {
        "ExtendedKeyUsageObjectIdentifier": (str, False),
        "ExtendedKeyUsageType": (str, False),
    }


class ExtendedKeyUsageList(AWSProperty):
    props = {
        "ExtendedKeyUsageList": ([ExtendedKeyUsage], False),
    }


class EdiPartyName(AWSProperty):
    props = {
        "NameAssigner": (str, True),
        "PartyName": (str, True),
    }


class OtherName(AWSProperty):
    props = {
        "TypeId": (str, True),
        "Value": (str, True),
    }


class Subject(AWSProperty):
    props = {
        "CommonName": (str, False),
        "Country": (str, False),
        "DistinguishedNameQualifier": (str, False),
        "GenerationQualifier": (str, False),
        "GivenName": (str, False),
        "Initials": (str, False),
        "Locality": (str, False),
        "Organization": (str, False),
        "OrganizationalUnit": (str, False),
        "Pseudonym": (str, False),
        "SerialNumber": (str, False),
        "State": (str, False),
        "Surname": (str, False),
        "Title": (str, False),
    }


class GeneralName(AWSProperty):
    props = {
        "DirectoryName": (Subject, False),
        "DnsName": (str, False),
        "EdiPartyName": (EdiPartyName, False),
        "IpAddress": (str, False),
        "OtherName": (OtherName, False),
        "RegisteredId": (str, False),
        "Rfc822Name": (str, False),
        "UniformResourceIdentifier": (str, False),
    }


class GeneralNameList(AWSProperty):
    props = {
        "GeneralNameList": ([GeneralName], False),
    }


class KeyUsage(AWSProperty):
    props = {
        "CRLSign": (boolean, False),
        "DataEncipherment": (boolean, False),
        "DecipherOnly": (boolean, False),
        "DigitalSignature": (boolean, False),
        "EncipherOnly": (boolean, False),
        "KeyAgreement": (boolean, False),
        "KeyCertSign": (boolean, False),
        "KeyEncipherment": (boolean, False),
        "NonRepudiation": (boolean, False),
    }


class Extensions(AWSProperty):
    props = {
        "CertificatePolicies": (CertificatePolicyList, False),
        "ExtendedKeyUsage": (ExtendedKeyUsageList, False),
        "KeyUsage": (KeyUsage, False),
        "SubjectAlternativeNames": (GeneralNameList, False),
    }


class ApiPassthrough(AWSProperty):
    props = {
        "Extensions": (Extensions, False),
        "Subject": (Subject, False),
    }


class Validity(AWSProperty):
    props = {
        "Type": (validate_validity_type, True),
        "Value": (integer, True),
    }


class Certificate(AWSObject):
    resource_type = "AWS::ACMPCA::Certificate"

    props = {
        "ApiPassthrough": (ApiPassthrough, False),
        "CertificateAuthorityArn": (str, True),
        "CertificateSigningRequest": (str, True),
        "SigningAlgorithm": (validate_signing_algorithm, True),
        "TemplateArn": (str, False),
        "Validity": (Validity, True),
        "ValidityNotBefore": (Validity, False),
    }


class CertificateAuthorityActivation(AWSObject):
    resource_type = "AWS::ACMPCA::CertificateAuthorityActivation"

    props = {
        "Certificate": (str, True),
        "CertificateAuthorityArn": (str, True),
        "CertificateChain": (str, False),
        "Status": (str, False),
    }


class AccessMethod(AWSProperty):
    props = {
        "AccessMethodType": (str, False),
        "CustomObjectIdentifier": (str, False),
    }


class AccessDescription(AWSProperty):
    props = {
        "AccessLocation": (GeneralName, True),
        "AccessMethod": (AccessMethod, True),
    }


class CsrExtensions(AWSProperty):
    props = {
        "KeyUsage": (KeyUsage, False),
        "SubjectInformationAccess": ([AccessDescription], False),
    }


class CrlConfiguration(AWSProperty):
    props = {
        "CustomCname": (str, False),
        "Enabled": (boolean, False),
        "ExpirationInDays": (integer, False),
        "S3BucketName": (str, False),
        "S3ObjectAcl": (str, False),
    }


class RevocationConfiguration(AWSProperty):
    props = {"CrlConfiguration": (CrlConfiguration, False)}


class Subject(AWSProperty):
    props = {
        "CommonName": (str, False),
        "Country": (str, False),
        "DistinguishedNameQualifier": (str, False),
        "GenerationQualifier": (str, False),
        "GivenName": (str, False),
        "Initials": (str, False),
        "Locality": (str, False),
        "Organization": (str, False),
        "OrganizationalUnit": (str, False),
        "Pseudonym": (str, False),
        "SerialNumber": (str, False),
        "State": (str, False),
        "Surname": (str, False),
        "Title": (str, False),
    }


class CertificateAuthority(AWSObject):
    resource_type = "AWS::ACMPCA::CertificateAuthority"

    props = {
        "CsrExtensions": (CsrExtensions, False),
        "KeyAlgorithm": (validate_key_algorithm, True),
        "KeyStorageSecurityStandard": (str, False),
        "RevocationConfiguration": (RevocationConfiguration, False),
        "SigningAlgorithm": (validate_signing_algorithm, True),
        "Subject": (Subject, True),
        "Tags": (Tags, False),
        "Type": (validate_certificateauthority_type, True),
    }



# Copyright (c) 2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags


class TransitGatewayRegistration(AWSObject):
    resource_type = "AWS::NetworkManager::TransitGatewayRegistration"

    props = {
        "GlobalNetworkId": (str, True),
        "TransitGatewayArn": (str, True),
    }


class Location(AWSProperty):
    props = {
        "Address": (str, False),
        "Latitude": (str, False),
        "Longitude": (str, False),
    }


class Site(AWSObject):
    resource_type = "AWS::NetworkManager::Site"

    props = {
        "Description": (str, False),
        "GlobalNetworkId": (str, True),
        "Location": (Location, False),
        "Tags": (Tags, False),
    }


class LinkAssociation(AWSObject):
    resource_type = "AWS::NetworkManager::LinkAssociation"

    props = {
        "DeviceId": (str, True),
        "GlobalNetworkId": (str, True),
        "LinkId": (str, True),
    }


class Bandwidth(AWSProperty):
    props = {
        "DownloadSpeed": (str, False),
        "UploadSpeed": (str, False),
    }


class Link(AWSObject):
    resource_type = "AWS::NetworkManager::Link"

    props = {
        "Bandwidth": (Bandwidth, True),
        "Description": (str, False),
        "GlobalNetworkId": (str, True),
        "Provider": (str, False),
        "SiteId": (str, True),
        "Tags": (Tags, False),
        "Type": (str, False),
    }


class GlobalNetwork(AWSObject):
    resource_type = "AWS::NetworkManager::GlobalNetwork"

    props = {
        "Description": (str, False),
        "Tags": (Tags, False),
    }


class Device(AWSObject):
    resource_type = "AWS::NetworkManager::Device"

    props = {
        "Description": (str, False),
        "GlobalNetworkId": (str, True),
        "Location": (str, False),
        "Model": (str, False),
        "SerialNumber": (str, False),
        "SiteId": (str, False),
        "Tags": (Tags, False),
        "Type": (str, False),
        "Vendor": (str, False),
    }


class CustomerGatewayAssociation(AWSObject):
    resource_type = "AWS::NetworkManager::CustomerGatewayAssociation"

    props = {
        "CustomerGatewayArn": (str, True),
        "DeviceId": (str, True),
        "GlobalNetworkId": (str, True),
        "LinkId": (str, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 40.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class Endpoint(AWSProperty):
    props = {
        "Address": (str, False),
        "Port": (str, False),
    }


class LoggingProperties(AWSProperty):
    props = {
        "BucketName": (str, True),
        "S3KeyPrefix": (str, False),
    }


class Cluster(AWSObject):
    resource_type = "AWS::Redshift::Cluster"

    props = {
        "AllowVersionUpgrade": (boolean, False),
        "AquaConfigurationStatus": (str, False),
        "AutomatedSnapshotRetentionPeriod": (integer, False),
        "AvailabilityZone": (str, False),
        "AvailabilityZoneRelocation": (boolean, False),
        "AvailabilityZoneRelocationStatus": (str, False),
        "Classic": (boolean, False),
        "ClusterIdentifier": (str, False),
        "ClusterParameterGroupName": (str, False),
        "ClusterSecurityGroups": ([str], False),
        "ClusterSubnetGroupName": (str, False),
        "ClusterType": (str, True),
        "ClusterVersion": (str, False),
        "DBName": (str, True),
        "DeferMaintenance": (boolean, False),
        "DeferMaintenanceDuration": (integer, False),
        "DeferMaintenanceEndTime": (str, False),
        "DeferMaintenanceIdentifier": (str, False),
        "DeferMaintenanceStartTime": (str, False),
        "DestinationRegion": (str, False),
        "ElasticIp": (str, False),
        "Encrypted": (boolean, False),
        "Endpoint": (Endpoint, False),
        "EnhancedVpcRouting": (boolean, False),
        "HsmClientCertificateIdentifier": (str, False),
        "HsmConfigurationIdentifier": (str, False),
        "IamRoles": ([str], False),
        "KmsKeyId": (str, False),
        "LoggingProperties": (LoggingProperties, False),
        "MaintenanceTrackName": (str, False),
        "ManualSnapshotRetentionPeriod": (integer, False),
        "MasterUserPassword": (str, True),
        "MasterUsername": (str, True),
        "NodeType": (str, True),
        "NumberOfNodes": (integer, False),
        "OwnerAccount": (str, False),
        "Port": (integer, False),
        "PreferredMaintenanceWindow": (str, False),
        "PubliclyAccessible": (boolean, False),
        "ResourceAction": (str, False),
        "RevisionTarget": (str, False),
        "RotateEncryptionKey": (boolean, False),
        "SnapshotClusterIdentifier": (str, False),
        "SnapshotCopyGrantName": (str, False),
        "SnapshotCopyManual": (boolean, False),
        "SnapshotCopyRetentionPeriod": (integer, False),
        "SnapshotIdentifier": (str, False),
        "Tags": (Tags, False),
        "VpcSecurityGroupIds": ([str], False),
    }


class AmazonRedshiftParameter(AWSProperty):
    props = {
        "ParameterName": (str, True),
        "ParameterValue": (str, True),
    }


class ClusterParameterGroup(AWSObject):
    resource_type = "AWS::Redshift::ClusterParameterGroup"

    props = {
        "Description": (str, True),
        "ParameterGroupFamily": (str, True),
        "Parameters": ([AmazonRedshiftParameter], False),
        "Tags": (Tags, False),
    }


class ClusterSecurityGroup(AWSObject):
    resource_type = "AWS::Redshift::ClusterSecurityGroup"

    props = {
        "Description": (str, True),
        "Tags": (Tags, False),
    }


class ClusterSecurityGroupIngress(AWSObject):
    resource_type = "AWS::Redshift::ClusterSecurityGroupIngress"

    props = {
        "CIDRIP": (str, False),
        "ClusterSecurityGroupName": (str, True),
        "EC2SecurityGroupName": (str, False),
        "EC2SecurityGroupOwnerId": (str, False),
    }


class ClusterSubnetGroup(AWSObject):
    resource_type = "AWS::Redshift::ClusterSubnetGroup"

    props = {
        "Description": (str, True),
        "SubnetIds": ([str], True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 8.1.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class ServiceAccountCredentials(AWSProperty):
    props = {
        "AccountName": (str, True),
        "AccountPassword": (str, True),
    }


class DirectoryConfig(AWSObject):
    resource_type = "AWS::AppStream::DirectoryConfig"

    props = {
        "DirectoryName": (str, True),
        "OrganizationalUnitDistinguishedNames": ([str], True),
        "ServiceAccountCredentials": (ServiceAccountCredentials, True),
    }


class ComputeCapacity(AWSProperty):
    props = {
        "DesiredInstances": (integer, True),
    }


class DomainJoinInfo(AWSProperty):
    props = {
        "DirectoryName": (str, False),
        "OrganizationalUnitDistinguishedName": (str, False),
    }


class VpcConfig(AWSProperty):
    props = {
        "SecurityGroupIds": ([str], False),
        "SubnetIds": ([str], False),
    }


class Fleet(AWSObject):
    resource_type = "AWS::AppStream::Fleet"

    props = {
        "ComputeCapacity": (ComputeCapacity, True),
        "Description": (str, False),
        "DisconnectTimeoutInSeconds": (integer, False),
        "DisplayName": (str, False),
        "DomainJoinInfo": (DomainJoinInfo, False),
        "EnableDefaultInternetAccess": (boolean, False),
        "FleetType": (str, False),
        "IamRoleArn": (str, False),
        "IdleDisconnectTimeoutInSeconds": (integer, False),
        "ImageArn": (str, False),
        "ImageName": (str, False),
        "InstanceType": (str, True),
        "MaxUserDurationInSeconds": (integer, False),
        "Name": (str, False),
        "StreamView": (str, False),
        "Tags": ((Tags, list), False),
        "VpcConfig": (VpcConfig, False),
    }


class AccessEndpoint(AWSProperty):
    props = {
        "EndpointType": (str, True),
        "VpceId": (str, True),
    }


class ImageBuilder(AWSObject):
    resource_type = "AWS::AppStream::ImageBuilder"

    props = {
        "AccessEndpoints": ([AccessEndpoint], False),
        "AppstreamAgentVersion": (str, False),
        "Description": (str, False),
        "DisplayName": (str, False),
        "DomainJoinInfo": (DomainJoinInfo, False),
        "EnableDefaultInternetAccess": (boolean, False),
        "IamRoleArn": (str, False),
        "ImageArn": (str, False),
        "ImageName": (str, False),
        "InstanceType": (str, True),
        "Name": (str, False),
        "Tags": ((Tags, list), False),
        "VpcConfig": (VpcConfig, False),
    }


class ApplicationSettings(AWSProperty):
    props = {
        "Enabled": (boolean, True),
        "SettingsGroup": (str, False),
    }


class StorageConnector(AWSProperty):
    props = {
        "ConnectorType": (str, True),
        "Domains": ([str], False),
        "ResourceIdentifier": (str, False),
    }


class UserSetting(AWSProperty):
    props = {
        "Action": (str, True),
        "Permission": (str, True),
    }


class Stack(AWSObject):
    resource_type = "AWS::AppStream::Stack"

    props = {
        "AccessEndpoints": ([AccessEndpoint], False),
        "ApplicationSettings": (ApplicationSettings, False),
        "AttributesToDelete": ([str], False),
        "DeleteStorageConnectors": (boolean, False),
        "Description": (str, False),
        "DisplayName": (str, False),
        "EmbedHostDomains": ([str], False),
        "FeedbackURL": (str, False),
        "Name": (str, False),
        "RedirectURL": (str, False),
        "StorageConnectors": ([StorageConnector], False),
        "Tags": ((Tags, list), False),
        "UserSettings": ([UserSetting], False),
    }


class StackFleetAssociation(AWSObject):
    resource_type = "AWS::AppStream::StackFleetAssociation"

    props = {
        "FleetName": (str, True),
        "StackName": (str, True),
    }


class StackUserAssociation(AWSObject):
    resource_type = "AWS::AppStream::StackUserAssociation"

    props = {
        "AuthenticationType": (str, True),
        "SendEmailNotification": (boolean, False),
        "StackName": (str, True),
        "UserName": (str, True),
    }


class User(AWSObject):
    resource_type = "AWS::AppStream::User"

    props = {
        "AuthenticationType": (str, True),
        "FirstName": (str, False),
        "LastName": (str, False),
        "MessageAction": (str, False),
        "UserName": (str, True),
    }



# Converted from Redshift.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import Equals, GetAtt, If, Join, Output, Parameter, Ref, Template
from troposphere.redshift import AmazonRedshiftParameter, Cluster, ClusterParameterGroup

t = Template()

t.set_version("2010-09-09")

t.set_description("AWS CloudFormation Sample Template: Redshift cluster")

dbname = t.add_parameter(
    Parameter(
        "DatabaseName",
        Description="The name of the first database to be created when the "
        "redshift cluster is created",
        Type="String",
        Default="defaultdb",
        AllowedPattern="([a-z]|[0-9])+",
    )
)

clustertype = t.add_parameter(
    Parameter(
        "ClusterType",
        Description="The type of the cluster",
        Type="String",
        Default="single-node",
        AllowedValues=["single-node", "multi-node"],
    )
)

numberofnodes = t.add_parameter(
    Parameter(
        "NumberOfNodes",
        Description="The number of compute nodes in the redshift cluster. "
        "When cluster type is specified as: 1) single-node, the NumberOfNodes "
        "parameter should be specified as 1, 2) multi-node, the NumberOfNodes "
        "parameter should be greater than 1",
        Type="Number",
        Default="1",
    )
)

nodetype = t.add_parameter(
    Parameter(
        "NodeType",
        Description="The node type to be provisioned for the redshift cluster",
        Type="String",
        Default="dw2.large",
    )
)

masterusername = t.add_parameter(
    Parameter(
        "MasterUsername",
        Description="The user name associated with the master user account for "
        "the redshift cluster that is being created",
        Type="String",
        Default="defaultuser",
        AllowedPattern="([a-z])([a-z]|[0-9])*",
        NoEcho=True,
    )
)

masteruserpassword = t.add_parameter(
    Parameter(
        "MasterUserPassword",
        Description="The password associated with the master user account for the "
        "redshift cluster that is being created.",
        Type="String",
        NoEcho=True,
    )
)

conditions = {
    "IsMultiNodeCluster": Equals(Ref("ClusterType"), "multi-node"),
}

for k in conditions:
    t.add_condition(k, conditions[k])

redshiftcluster = t.add_resource(
    Cluster(
        "RedshiftCluster",
        ClusterType=Ref("ClusterType"),
        NumberOfNodes=If(
            "IsMultiNodeCluster", Ref("NumberOfNodes"), Ref("AWS::NoValue")
        ),
        NodeType=Ref("NodeType"),
        DBName=Ref("DatabaseName"),
        MasterUsername=Ref("MasterUsername"),
        MasterUserPassword=Ref("MasterUserPassword"),
        ClusterParameterGroupName=Ref("RedshiftClusterParameterGroup"),
        DeletionPolicy="Snapshot",
    )
)

amazonredshiftparameter1 = AmazonRedshiftParameter(
    "AmazonRedshiftParameter1",
    ParameterName="enable_user_activity_logging",
    ParameterValue="true",
)

redshiftclusterparametergroup = t.add_resource(
    ClusterParameterGroup(
        "RedshiftClusterParameterGroup",
        Description="Cluster parameter group",
        ParameterGroupFamily="redshift-1.0",
        Parameters=[amazonredshiftparameter1],
    )
)

t.add_output(
    Output(
        "ClusterEndpoint",
        Value=Join(
            ":",
            [
                GetAtt(redshiftcluster, "Endpoint.Address"),
                GetAtt(redshiftcluster, "Endpoint.Port"),
            ],
        ),
    )
)

print(t.to_json())



# Copyright (c) 2015, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import (
    boolean,
    compliance_level,
    integer,
    json_checker,
    notification_event,
    notification_type,
    operating_system,
    s3_bucket_name,
    task_type,
)


class NotificationConfig(AWSProperty):
    props = {
        "NotificationArn": (str, False),
        "NotificationEvents": (notification_event, False),
        "NotificationType": (notification_type, False),
    }


class LoggingInfo(AWSProperty):
    props = {
        "Region": (str, True),
        "S3Bucket": (s3_bucket_name, True),
        "S3Prefix": (str, False),
    }


class MaintenanceWindowAutomationParameters(AWSProperty):
    props = {
        "DocumentVersion": (str, False),
        "Parameters": (dict, False),
    }


class MaintenanceWindowLambdaParameters(AWSProperty):
    props = {
        "ClientContext": (str, False),
        "Payload": (json_checker, False),
        "Qualifier": (str, False),
    }


class MaintenanceWindowRunCommandParameters(AWSProperty):
    props = {
        "Comment": (str, False),
        "DocumentHash": (str, False),
        "DocumentHashType": (str, False),
        "NotificationConfig": (NotificationConfig, False),
        "OutputS3BucketName": (s3_bucket_name, False),
        "OutputS3KeyPrefix": (str, False),
        "Parameters": (dict, False),
        "ServiceRoleArn": (str, False),
        "TimeoutSeconds": (integer, False),
    }


class MaintenanceWindowStepFunctionsParameters(AWSProperty):
    props = {
        "Input": (str, False),
        "Name": (str, False),
    }


class PatchFilter(AWSProperty):
    props = {
        "Key": (str, True),
        "Values": ([str], True),
    }


class PatchFilterGroup(AWSProperty):
    props = {
        "PatchFilters": ([PatchFilter], False),
    }


class Rule(AWSProperty):
    props = {
        "ApproveAfterDays": (integer, False),
        "ComplianceLevel": (compliance_level, False),
        "EnableNonSecurity": (boolean, False),
        "PatchFilterGroup": (PatchFilterGroup, False),
    }


class RuleGroup(AWSProperty):
    props = {
        "PatchRules": ([Rule], False),
    }


class TaskInvocationParameters(AWSProperty):
    props = {
        "MaintenanceWindowAutomationParameters": (
            MaintenanceWindowAutomationParameters,
            False,
        ),
        "MaintenanceWindowLambdaParameters": (MaintenanceWindowLambdaParameters, False),
        "MaintenanceWindowRunCommandParameters": (
            MaintenanceWindowRunCommandParameters,
            False,
        ),
        "MaintenanceWindowStepFunctionsParameters": (
            MaintenanceWindowStepFunctionsParameters,
            False,
        ),
    }


class Targets(AWSProperty):
    props = {
        "Key": (str, True),
        "Values": ([str], True),
    }


class S3OutputLocation(AWSProperty):
    props = {
        "OutputS3BucketName": (str, False),
        "OutputS3KeyPrefix": (str, False),
    }


class InstanceAssociationOutputLocation(AWSProperty):
    props = {
        "S3Location": (S3OutputLocation, False),
    }


class Association(AWSObject):
    resource_type = "AWS::SSM::Association"

    props = {
        "AssociationName": (str, False),
        "DocumentVersion": (str, False),
        "InstanceId": (str, False),
        "Name": (str, True),
        "OutputLocation": (InstanceAssociationOutputLocation, False),
        "Parameters": (dict, False),
        "ScheduleExpression": (str, False),
        "Targets": ([Targets], False),
        "WaitForSuccessTimeoutSeconds": (integer, False),
    }


class AttachmentsSource(AWSProperty):
    props = {
        "Key": (str, False),
        "Name": (str, False),
        "Values": ([str], False),
    }


class DocumentRequires(AWSProperty):
    props = {
        "Name": (str, False),
        "Version": (str, False),
    }


class Document(AWSObject):
    resource_type = "AWS::SSM::Document"

    props = {
        "Attachments": ([AttachmentsSource], False),
        "Content": (dict, True),
        "DocumentFormat": (str, False),
        "DocumentType": (str, False),
        "Name": (str, False),
        "Requires": ([DocumentRequires], False),
        "Tags": (Tags, False),
        "TargetType": (str, False),
        "VersionName": (str, False),
    }


class MaintenanceWindow(AWSObject):
    resource_type = "AWS::SSM::MaintenanceWindow"

    props = {
        "AllowUnassociatedTargets": (boolean, True),
        "Cutoff": (integer, True),
        "Description": (str, False),
        "Duration": (integer, True),
        "EndDate": (str, False),
        "Name": (str, True),
        "Schedule": (str, True),
        "ScheduleTimezone": (str, False),
        "StartDate": (str, False),
        "Tags": (Tags, False),
    }


class MaintenanceWindowTarget(AWSObject):
    resource_type = "AWS::SSM::MaintenanceWindowTarget"

    props = {
        "Description": (str, False),
        "Name": (str, False),
        "OwnerInformation": (str, False),
        "ResourceType": (str, True),
        "Targets": ([Targets], True),
        "WindowId": (str, True),
    }


class MaintenanceWindowTask(AWSObject):
    resource_type = "AWS::SSM::MaintenanceWindowTask"

    props = {
        "Description": (str, False),
        "LoggingInfo": (LoggingInfo, False),
        "MaxConcurrency": (str, False),
        "MaxErrors": (str, True),
        "Name": (str, False),
        "Priority": (integer, True),
        "ServiceRoleArn": (str, True),
        "Targets": ([Targets], True),
        "TaskArn": (str, True),
        "TaskInvocationParameters": (TaskInvocationParameters, False),
        "TaskParameters": (dict, False),
        "TaskType": (task_type, True),
        "WindowId": (str, False),
    }


class Parameter(AWSObject):
    resource_type = "AWS::SSM::Parameter"

    props = {
        "AllowedPattern": (str, False),
        "DataType": (str, False),
        "Description": (str, False),
        "Name": (str, False),
        "Policies": (str, False),
        "Tags": (dict, False),
        "Tier": (str, False),
        "Type": (str, True),
        "Value": (str, True),
    }


class PatchSource(AWSProperty):
    props = {
        "Configuration": (str, False),
        "Name": (str, False),
        "Products": ([str], False),
    }


class PatchBaseline(AWSObject):
    resource_type = "AWS::SSM::PatchBaseline"

    props = {
        "ApprovalRules": (RuleGroup, False),
        "ApprovedPatches": ([str], False),
        "ApprovedPatchesComplianceLevel": (compliance_level, False),
        "ApprovedPatchesEnableNonSecurity": (boolean, False),
        "Description": (str, False),
        "GlobalFilters": (PatchFilterGroup, False),
        "Name": (str, True),
        "OperatingSystem": (operating_system, False),
        "PatchGroups": ([str], False),
        "RejectedPatches": ([str], False),
        "RejectedPatchesAction": (str, False),
        "Sources": ([PatchSource], False),
        "Tags": (Tags, False),
    }


class AwsOrganizationsSource(AWSProperty):
    props = {
        "OrganizationalUnits": ([str], False),
        "OrganizationSourceType": (str, True),
    }


class SyncSource(AWSProperty):
    props = {
        "AwsOrganizationsSource": (AwsOrganizationsSource, False),
        "IncludeFutureRegions": (boolean, False),
        "SourceRegions": ([str], True),
        "SourceType": (str, True),
    }


class ResourceDataSync(AWSObject):
    resource_type = "AWS::SSM::ResourceDataSync"

    props = {
        "BucketName": (str, True),
        "BucketPrefix": (str, False),
        "BucketRegion": (str, True),
        "KMSKeyArn": (str, False),
        "SyncFormat": (str, True),
        "SyncName": (str, True),
        "SyncSource": (SyncSource, False),
        "SyncType": (str, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 25.0.0


from troposphere import Tags

from . import AWSObject


class Domain(AWSObject):
    resource_type = "AWS::CodeArtifact::Domain"

    props = {
        "DomainName": (str, True),
        "EncryptionKey": (str, False),
        "PermissionsPolicyDocument": (dict, False),
        "Tags": (Tags, False),
    }


class Repository(AWSObject):
    resource_type = "AWS::CodeArtifact::Repository"

    props = {
        "Description": (str, False),
        "DomainName": (str, True),
        "DomainOwner": (str, False),
        "ExternalConnections": ([str], False),
        "PermissionsPolicyDocument": (dict, False),
        "RepositoryName": (str, True),
        "Tags": (Tags, False),
        "Upstreams": ([str], False),
    }



from troposphere import Template
from troposphere.codebuild import Artifacts, Environment, Project, Source

template = Template()
template.set_version("2010-09-09")

artifacts = Artifacts(Type="NO_ARTIFACTS")

environment = Environment(
    ComputeType="BUILD_GENERAL1_SMALL",
    Image="aws/codebuild/java:openjdk-8",
    Type="LINUX_CONTAINER",
    EnvironmentVariables=[{"Name": "APP_NAME", "Value": "demo"}],
)

source = Source(
    Location="codebuild-demo-test/0123ab9a371ebf0187b0fe5614fbb72c", Type="S3"
)

project = Project(
    "DemoProject",
    Artifacts=artifacts,
    Environment=environment,
    Name="DemoProject",
    ServiceRole="arn:aws:iam::0123456789:role/codebuild-role",
    Source=source,
)
template.add_resource(project)

print(template.to_json())



# Copyright (c) 2014, Yuta Okamoto <okapies@gmail.com>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty, Tags
from .validators import boolean, integer, mutually_exclusive


class Source(AWSProperty):
    props = {
        "Password": (str, False),
        "Revision": (str, False),
        "SshKey": (str, False),
        "Type": (str, False),
        "Url": (str, False),
        "Username": (str, False),
    }


class SslConfiguration(AWSProperty):
    props = {
        "Certificate": (str, True),
        "Chain": (str, False),
        "PrivateKey": (str, True),
    }


class ChefConfiguration(AWSProperty):
    props = {
        "BerkshelfVersion": (str, False),
        "ManageBerkshelf": (boolean, False),
    }


class Recipes(AWSProperty):
    props = {
        "Configure": ([str], False),
        "Deploy": ([str], False),
        "Setup": ([str], False),
        "Shutdown": ([str], False),
        "Undeploy": ([str], False),
    }


def validate_volume_type(volume_type):
    volume_types = ("standard", "io1", "gp2")
    if volume_type not in volume_types:
        raise ValueError(
            "VolumeType (given: %s) must be one of: %s"
            % (volume_type, ", ".join(volume_types))
        )
    return volume_type


class VolumeConfiguration(AWSProperty):
    props = {
        "Encrypted": (boolean, False),
        "Iops": (integer, False),
        "MountPoint": (str, True),
        "NumberOfDisks": (integer, True),
        "RaidLevel": (integer, False),
        "Size": (integer, True),
        "VolumeType": (validate_volume_type, False),
    }

    def validate(self):
        volume_type = self.properties.get("VolumeType")
        iops = self.properties.get("Iops")
        if volume_type == "io1" and not iops:
            raise ValueError("Must specify Iops if VolumeType is 'io1'.")
        if volume_type != "io1" and iops:
            raise ValueError("Cannot specify Iops if VolumeType is not 'io1'.")


class StackConfigurationManager(AWSProperty):
    props = {
        "Name": (str, False),
        "Version": (str, False),
    }


class TimeBasedAutoScaling(AWSProperty):
    props = {
        "Monday": (dict, False),
        "Tuesday": (dict, False),
        "Wednesday": (dict, False),
        "Thursday": (dict, False),
        "Friday": (dict, False),
        "Saturday": (dict, False),
        "Sunday": (dict, False),
    }


class AutoScalingThresholds(AWSProperty):
    props = {
        "CpuThreshold": (float, False),
        "IgnoreMetricsTime": (integer, False),
        "InstanceCount": (integer, False),
        "LoadThreshold": (float, False),
        "MemoryThreshold": (float, False),
        "ThresholdsWaitTime": (integer, False),
    }


class Environment(AWSProperty):
    props = {
        "Key": (str, True),
        "Secure": (bool, False),
        "Value": (str, True),
    }


class LoadBasedAutoScaling(AWSProperty):
    props = {
        "DownScaling": (AutoScalingThresholds, False),
        "Enable": (bool, False),
        "UpScaling": (AutoScalingThresholds, False),
    }


def validate_data_source_type(data_source_type):
    data_source_types = (
        "AutoSelectOpsworksMysqlInstance",
        "OpsworksMysqlInstance",
        "RdsDbInstance",
    )
    if data_source_type not in data_source_types:
        raise ValueError(
            "Type (given: %s) must be one of: %s"
            % (data_source_type, ", ".join(data_source_types))
        )
    return data_source_type


class DataSource(AWSProperty):
    props = {
        "Arn": (str, False),
        "DatabaseName": (str, False),
        "Type": (validate_data_source_type, False),
    }


class App(AWSObject):
    resource_type = "AWS::OpsWorks::App"

    props = {
        "AppSource": (Source, False),
        "Attributes": (dict, False),
        "DataSources": ([DataSource], False),
        "Description": (str, False),
        "Domains": ([str], False),
        "EnableSsl": (boolean, False),
        "Environment": ([Environment], False),
        "Name": (str, True),
        "Shortname": (str, False),
        "SslConfiguration": (SslConfiguration, False),
        "StackId": (str, True),
        "Type": (str, True),
    }


class ElasticLoadBalancerAttachment(AWSObject):
    resource_type = "AWS::OpsWorks::ElasticLoadBalancerAttachment"

    props = {
        "ElasticLoadBalancerName": (str, True),
        "LayerId": (str, True),
        "Tags": ((Tags, list), False),
    }


class EbsBlockDevice(AWSProperty):
    props = {
        "DeleteOnTermination": (boolean, False),
        "Iops": (integer, False),
        "SnapshotId": (str, False),
        "VolumeSize": (integer, False),
        "VolumeType": (str, False),
    }


class BlockDeviceMapping(AWSProperty):
    props = {
        "DeviceName": (str, False),
        "Ebs": (EbsBlockDevice, False),
        "NoDevice": (str, False),
        "VirtualName": (str, False),
    }

    def validate(self):
        conds = [
            "Ebs",
            "VirtualName",
        ]
        mutually_exclusive(self.__class__.__name__, self.properties, conds)


class Instance(AWSObject):
    resource_type = "AWS::OpsWorks::Instance"

    props = {
        "AgentVersion": (str, False),
        "AmiId": (str, False),
        "Architecture": (str, False),
        "AutoScalingType": (str, False),
        "AvailabilityZone": (str, False),
        "BlockDeviceMappings": ([BlockDeviceMapping], False),
        "EbsOptimized": (boolean, False),
        "ElasticIps": ([str], False),
        "Hostname": (str, False),
        "InstallUpdatesOnBoot": (boolean, False),
        "InstanceType": (str, True),
        "LayerIds": ([str], True),
        "Os": (str, False),
        "RootDeviceType": (str, False),
        "SshKeyName": (str, False),
        "StackId": (str, True),
        "SubnetId": (str, False),
        "Tenancy": (str, False),
        "TimeBasedAutoScaling": (TimeBasedAutoScaling, False),
        "VirtualizationType": (str, False),
        "Volumes": ([str], False),
    }


class ShutdownEventConfiguration(AWSProperty):
    props = {
        "DelayUntilElbConnectionsDrained": (boolean, False),
        "ExecutionTimeout": (integer, False),
    }


class LifeCycleConfiguration(AWSProperty):
    props = {
        "ShutdownEventConfiguration": (ShutdownEventConfiguration, False),
    }


class Layer(AWSObject):
    resource_type = "AWS::OpsWorks::Layer"

    props = {
        "Attributes": (dict, False),
        "AutoAssignElasticIps": (boolean, True),
        "AutoAssignPublicIps": (boolean, True),
        "CustomInstanceProfileArn": (str, False),
        "CustomJson": ((str, dict), False),
        "CustomRecipes": (Recipes, False),
        "CustomSecurityGroupIds": ([str], False),
        "EnableAutoHealing": (boolean, True),
        "InstallUpdatesOnBoot": (boolean, False),
        "LifecycleEventConfiguration": (LifeCycleConfiguration, False),
        "LoadBasedAutoScaling": (LoadBasedAutoScaling, False),
        "Name": (str, True),
        "Packages": ([str], False),
        "Shortname": (str, True),
        "StackId": (str, True),
        "Type": (str, True),
        "VolumeConfigurations": ([VolumeConfiguration], False),
    }


class RdsDbInstance(AWSProperty):
    props = {
        "DbPassword": (str, True),
        "DbUser": (str, True),
        "RdsDbInstanceArn": (str, True),
    }


class ElasticIp(AWSProperty):
    props = {
        "Ip": (str, True),
        "Name": (str, False),
    }


class Stack(AWSObject):
    resource_type = "AWS::OpsWorks::Stack"

    props = {
        "AgentVersion": (str, False),
        "Attributes": (dict, False),
        "ChefConfiguration": (ChefConfiguration, False),
        "CloneAppIds": ([str], False),
        "ClonePermissions": (boolean, False),
        "ConfigurationManager": (StackConfigurationManager, False),
        "CustomCookbooksSource": (Source, False),
        "CustomJson": ((str, dict), False),
        "DefaultAvailabilityZone": (str, False),
        "DefaultInstanceProfileArn": (str, True),
        "DefaultOs": (str, False),
        "DefaultRootDeviceType": (str, False),
        "DefaultSshKeyName": (str, False),
        "DefaultSubnetId": (str, False),
        "EcsClusterArn": (str, False),
        "ElasticIps": ([ElasticIp], False),
        "HostnameTheme": (str, False),
        "Name": (str, True),
        "RdsDbInstances": ([RdsDbInstance], False),
        "ServiceRoleArn": (str, True),
        "SourceStackId": (str, False),
        "Tags": ((Tags, list), False),
        "UseCustomCookbooks": (boolean, False),
        "UseOpsworksSecurityGroups": (boolean, False),
        "VpcId": (str, False),
    }

    def validate(self):
        if "VpcId" in self.properties and "DefaultSubnetId" not in self.properties:
            raise ValueError("Using VpcId requires DefaultSubnetId to be" "specified")
        return True


class UserProfile(AWSObject):
    resource_type = "AWS::OpsWorks::UserProfile"

    props = {
        "AllowSelfManagement": (boolean, False),
        "IamUserArn": (str, True),
        "SshPublicKey": (str, False),
        "SshUsername": (str, False),
    }


class Volume(AWSObject):
    resource_type = "AWS::OpsWorks::Volume"

    props = {
        "Ec2VolumeId": (str, True),
        "MountPoint": (str, False),
        "Name": (str, False),
        "StackId": (str, True),
    }


class EngineAttribute(AWSProperty):
    props = {
        "Name": (str, False),
        "Value": (str, False),
    }


class Server(AWSObject):
    resource_type = "AWS::OpsWorksCM::Server"

    props = {
        "AssociatePublicIpAddress": (boolean, False),
        "BackupId": (str, False),
        "BackupRetentionCount": (integer, False),
        "CustomCertificate": (str, False),
        "CustomDomain": (str, False),
        "CustomPrivateKey": (str, False),
        "DisableAutomatedBackup": (boolean, False),
        "Engine": (str, False),
        "EngineAttributes": ([EngineAttribute], False),
        "EngineModel": (str, False),
        "EngineVersion": (str, False),
        "InstanceProfileArn": (str, True),
        "InstanceType": (str, True),
        "KeyPair": (str, False),
        "PreferredBackupWindow": (str, False),
        "PreferredMaintenanceWindow": (str, False),
        "SecurityGroupIds": ([str], False),
        "ServerName": (str, False),
        "ServiceRoleArn": (str, True),
        "SubnetIds": ([str], False),
        "Tags": ((Tags, list), False),
    }



# Converted from IAM_Policies_SNS_Publish_To_SQS.template located at:
# http://aws.amazon.com/cloudformation/aws-cloudformation-templates/

from troposphere import GetAtt, Output, Ref, Template
from troposphere.sns import Subscription, Topic
from troposphere.sqs import Queue, QueuePolicy

t = Template()

t.set_description(
    "AWS CloudFormation Sample Template: This template "
    "demonstrates the creation of a DynamoDB table."
)

sqsqueue = t.add_resource(Queue("SQSQueue"))

snstopic = t.add_resource(
    Topic(
        "SNSTopic",
        Subscription=[Subscription(Protocol="sqs", Endpoint=GetAtt(sqsqueue, "Arn"))],
    )
)

t.add_output(
    Output(
        "QueueArn",
        Value=GetAtt(sqsqueue, "Arn"),
        Description="ARN of SQS Queue",
    )
)

t.add_resource(
    QueuePolicy(
        "AllowSNS2SQSPolicy",
        Queues=[Ref(sqsqueue)],
        PolicyDocument={
            "Version": "2008-10-17",
            "Id": "PublicationPolicy",
            "Statement": [
                {
                    "Sid": "Allow-SNS-SendMessage",
                    "Effect": "Allow",
                    "Principal": {"AWS": "*"},
                    "Action": ["sqs:SendMessage"],
                    "Resource": GetAtt(sqsqueue, "Arn"),
                    "Condition": {"ArnEquals": {"aws:SourceArn": Ref(snstopic)}},
                }
            ],
        },
    )
)

print(t.to_json())



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.1.0


from . import AWSObject, AWSProperty


class FederationParameters(AWSProperty):
    props = {
        "ApplicationCallBackURL": (str, False),
        "AttributeMap": (dict, False),
        "FederationProviderName": (str, False),
        "FederationURN": (str, False),
        "SamlMetadataDocument": (str, False),
        "SamlMetadataURL": (str, False),
    }


class Environment(AWSObject):
    resource_type = "AWS::FinSpace::Environment"

    props = {
        "Description": (str, False),
        "FederationMode": (str, False),
        "FederationParameters": (FederationParameters, False),
        "KmsKeyId": (str, False),
        "Name": (str, True),
    }



# Converted from AWS OpsWorks Snippets located at:
# http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/quickref-opsworks.html

from troposphere import GetAZs, Join, Parameter, Ref, Template
from troposphere.elasticloadbalancing import HealthCheck, LoadBalancer
from troposphere.opsworks import (
    App,
    ElasticLoadBalancerAttachment,
    Instance,
    Layer,
    Recipes,
    Source,
    Stack,
    VolumeConfiguration,
)

template = Template()

template.set_version("2010-09-09")

ServiceRole = template.add_parameter(
    Parameter(
        "ServiceRole",
        Default="aws-opsworks-service-role",
        Description="The OpsWorks service role",
        Type="String",
        MinLength="1",
        MaxLength="64",
        AllowedPattern="[a-zA-Z][a-zA-Z0-9-]*",
        ConstraintDescription="must begin with a letter and contain only "
        + "alphanumeric characters.",
    )
)

InstanceRole = template.add_parameter(
    Parameter(
        "InstanceRole",
        Default="aws-opsworks-ec2-role",
        Description="The OpsWorks instance role",
        Type="String",
        MinLength="1",
        MaxLength="64",
        AllowedPattern="[a-zA-Z][a-zA-Z0-9-]*",
        ConstraintDescription="must begin with a letter and contain only "
        + "alphanumeric characters.",
    )
)

AppName = template.add_parameter(
    Parameter(
        "AppName",
        Default="myapp",
        Description="The app name",
        Type="String",
        MinLength="1",
        MaxLength="64",
        AllowedPattern="[a-zA-Z][a-zA-Z0-9]*",
        ConstraintDescription="must begin with a letter and contain only "
        + "alphanumeric characters.",
    )
)

MysqlRootPassword = template.add_parameter(
    Parameter(
        "MysqlRootPassword",
        Description="MysqlRootPassword",
        NoEcho=True,
        Type="String",
    )
)

myStack = template.add_resource(
    Stack(
        "myStack",
        Name=Ref("AWS::StackName"),
        ServiceRoleArn=Join(
            "", ["arn:aws:iam::", Ref("AWS::AccountId"), ":role/", Ref(ServiceRole)]
        ),
        DefaultInstanceProfileArn=Join(
            "",
            [
                "arn:aws:iam::",
                Ref("AWS::AccountId"),
                ":instance-profile/",
                Ref(InstanceRole),
            ],
        ),
        UseCustomCookbooks=True,
        CustomCookbooksSource=Source(
            Type="git",
            Url="git://github.com/amazonwebservices/"
            + "opsworks-example-cookbooks.git",
        ),
    )
)

myLayer = template.add_resource(
    Layer(
        "myLayer",
        StackId=Ref(myStack),
        Type="php-app",
        Shortname="php-app",
        EnableAutoHealing=True,
        AutoAssignElasticIps=False,
        AutoAssignPublicIps=True,
        Name="MyPHPApp",
        CustomRecipes=Recipes(
            Configure=["phpapp::appsetup"],
        ),
    )
)

DBLayer = template.add_resource(
    Layer(
        "DBLayer",
        StackId=Ref(myStack),
        Type="db-master",
        Shortname="db-layer",
        EnableAutoHealing=True,
        AutoAssignElasticIps=False,
        AutoAssignPublicIps=True,
        Name="MyMySQL",
        CustomRecipes=Recipes(Setup=["phpapp::dbsetup"]),
        Attributes={
            "MysqlRootPassword": Ref(MysqlRootPassword),
            "MysqlRootPasswordUbiquitous": "true",
        },
        VolumeConfigurations=[
            VolumeConfiguration(
                MountPoint="/vol/mysql",
                NumberOfDisks=1,
                Size=10,
            )
        ],
    )
)

ELB = template.add_resource(
    LoadBalancer(
        "ELB",
        AvailabilityZones=GetAZs(""),
        Listeners=[
            {
                "LoadBalancerPort": "80",
                "InstancePort": "80",
                "Protocol": "HTTP",
                "InstanceProtocol": "HTTP",
            }
        ],
        HealthCheck=HealthCheck(
            Target="HTTP:80/",
            HealthyThreshold="2",
            UnhealthyThreshold="10",
            Interval="30",
            Timeout="5",
        ),
    )
)

ELBAttachment = template.add_resource(
    ElasticLoadBalancerAttachment(
        "ELBAttachment",
        ElasticLoadBalancerName=Ref(ELB),
        LayerId=Ref(myLayer),
    )
)

myAppInstance1 = template.add_resource(
    Instance(
        "myAppInstance1",
        StackId=Ref(myStack),
        LayerIds=[Ref(myLayer)],
        InstanceType="m1.small",
    )
)

myAppInstance2 = template.add_resource(
    Instance(
        "myAppInstance2",
        StackId=Ref(myStack),
        LayerIds=[Ref(myLayer)],
        InstanceType="m1.small",
    )
)

myDBInstance = template.add_resource(
    Instance(
        "myDBInstance",
        StackId=Ref(myStack),
        LayerIds=[Ref(DBLayer)],
        InstanceType="m1.small",
    )
)

myApp = template.add_resource(
    App(
        "myApp",
        StackId=Ref(myStack),
        Type="php",
        Name=Ref(AppName),
        AppSource=Source(
            Type="git",
            Url="git://github.com/amazonwebservices/"
            + "opsworks-demo-php-simple-app.git",
            Revision="version2",
        ),
        Attributes={
            "DocumentRoot": "web",
        },
    )
)

print(template.to_json())



# Copyright (c) 2012-2013, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSHelperFn, AWSObject, AWSProperty, If, Tags
from .validators import boolean, double, integer


def attribute_type_validator(x):
    valid_types = ["S", "N", "B"]
    if x not in valid_types:
        raise ValueError("AttributeType must be one of: %s" % ", ".join(valid_types))
    return x


def key_type_validator(x):
    valid_types = ["HASH", "RANGE"]
    if x not in valid_types:
        raise ValueError("KeyType must be one of: %s" % ", ".join(valid_types))
    return x


def projection_type_validator(x):
    valid_types = ["KEYS_ONLY", "INCLUDE", "ALL"]
    if x not in valid_types:
        raise ValueError("ProjectionType must be one of: %s" % ", ".join(valid_types))
    return x


def billing_mode_validator(x):
    valid_modes = ["PROVISIONED", "PAY_PER_REQUEST"]
    if x not in valid_modes:
        raise ValueError(
            "Table billing mode must be one of: %s" % ", ".join(valid_modes)
        )
    return x


class AttributeDefinition(AWSProperty):
    props = {
        "AttributeName": (str, True),
        "AttributeType": (attribute_type_validator, True),
    }


class ContributorInsightsSpecification(AWSProperty):
    props = {
        "Enabled": (boolean, True),
    }


class KeySchema(AWSProperty):
    props = {"AttributeName": (str, True), "KeyType": (key_type_validator, True)}


class Key(KeySchema):
    """For backwards compatibility."""

    pass


class KinesisStreamSpecification(AWSProperty):
    props = {"StreamArn": (str, True)}


class ProvisionedThroughput(AWSProperty):
    props = {
        "ReadCapacityUnits": (int, True),
        "WriteCapacityUnits": (int, True),
    }


class Projection(AWSProperty):
    props = {
        "NonKeyAttributes": ([str], False),
        "ProjectionType": (projection_type_validator, False),
    }


class SSESpecification(AWSProperty):
    props = {
        "KMSMasterKeyId": (str, False),
        "SSEEnabled": (boolean, True),
        "SSEType": (str, False),
    }


class GlobalSecondaryIndex(AWSProperty):
    props = {
        "IndexName": (str, True),
        "KeySchema": ([KeySchema], True),
        "Projection": (Projection, True),
        "ProvisionedThroughput": (ProvisionedThroughput, False),
    }


class LocalSecondaryIndex(AWSProperty):
    props = {
        "IndexName": (str, True),
        "KeySchema": ([KeySchema], True),
        "Projection": (Projection, True),
    }


class PointInTimeRecoverySpecification(AWSProperty):
    props = {
        "PointInTimeRecoveryEnabled": (boolean, False),
    }


class StreamSpecification(AWSProperty):
    props = {
        "StreamViewType": (str, True),
    }


class TimeToLiveSpecification(AWSProperty):
    props = {
        "AttributeName": (str, True),
        "Enabled": (boolean, True),
    }


class TargetTrackingScalingPolicyConfiguration(AWSProperty):
    props = {
        "DisableScaleIn": (boolean, False),
        "ScaleInCooldown": (integer, False),
        "ScaleOutCooldown": (integer, False),
        "TargetValue": (double, True),
    }


class CapacityAutoScalingSettings(AWSProperty):
    props = {
        "MaxCapacity": (integer, True),
        "MinCapacity": (integer, True),
        "SeedCapacity": (integer, False),
        "TargetTrackingScalingPolicyConfiguration": (
            TargetTrackingScalingPolicyConfiguration,
            True,
        ),
    }


class ReadProvisionedThroughputSettings(AWSProperty):
    props = {
        "ReadCapacityAutoScalingSettings": (CapacityAutoScalingSettings, False),
        "ReadCapacityUnits": (integer, False),
    }


class ReplicaGlobalSecondaryIndexSpecification(AWSProperty):
    props = {
        "ContributorInsightsSpecification": (ContributorInsightsSpecification, False),
        "IndexName": (str, True),
        "ReadProvisionedThroughputSettings": (ReadProvisionedThroughputSettings, False),
    }


class ReplicaSSESpecification(AWSProperty):
    props = {
        "KMSMasterKeyId": (str, True),
    }


class ReplicaSpecification(AWSProperty):
    props = {
        "ContributorInsightsSpecification": (ContributorInsightsSpecification, False),
        "GlobalSecondaryIndexes": ([ReplicaGlobalSecondaryIndexSpecification], False),
        "PointInTimeRecoverySpecification": (PointInTimeRecoverySpecification, False),
        "ReadProvisionedThroughputSettings": (ReadProvisionedThroughputSettings, False),
        "Region": (str, True),
        "SSESpecification": (ReplicaSSESpecification, False),
        "Tags": (Tags, False),
    }


class WriteProvisionedThroughputSettings(AWSProperty):
    props = {
        "WriteCapacityAutoScalingSettings": (CapacityAutoScalingSettings, False),
    }


class GlobalTable(AWSObject):
    resource_type = "AWS::DynamoDB::GlobalTable"

    props = {
        "AttributeDefinitions": ([AttributeDefinition], True),
        "BillingMode": (str, False),
        "GlobalSecondaryIndexes": ([GlobalSecondaryIndex], False),
        "KeySchema": ([KeySchema], True),
        "LocalSecondaryIndexes": ([LocalSecondaryIndex], False),
        "Replicas": ([ReplicaSpecification], True),
        "SSESpecification": (SSESpecification, False),
        "StreamSpecification": (StreamSpecification, False),
        "TableName": (str, False),
        "TimeToLiveSpecification": (TimeToLiveSpecification, False),
        "WriteProvisionedThroughputSettings": (
            WriteProvisionedThroughputSettings,
            False,
        ),
    }


class Table(AWSObject):
    resource_type = "AWS::DynamoDB::Table"

    props = {
        "AttributeDefinitions": ([AttributeDefinition], True),
        "BillingMode": (billing_mode_validator, False),
        "ContributorInsightsSpecification": (ContributorInsightsSpecification, False),
        "GlobalSecondaryIndexes": ([GlobalSecondaryIndex], False),
        "KeySchema": ([KeySchema], True),
        "KinesisStreamSpecification": (KinesisStreamSpecification, False),
        "LocalSecondaryIndexes": ([LocalSecondaryIndex], False),
        "PointInTimeRecoverySpecification": (PointInTimeRecoverySpecification, False),
        "ProvisionedThroughput": (ProvisionedThroughput, False),
        "SSESpecification": (SSESpecification, False),
        "StreamSpecification": (StreamSpecification, False),
        "TableName": (str, False),
        "Tags": (Tags, False),
        "TimeToLiveSpecification": (TimeToLiveSpecification, False),
    }

    def validate(self):
        billing_mode = self.properties.get("BillingMode", "PROVISIONED")
        indexes = self.properties.get("GlobalSecondaryIndexes", [])
        tput_props = [self.properties]
        tput_props.extend(
            [x.properties for x in indexes if not isinstance(x, AWSHelperFn)]
        )

        def check_if_all(name, props):
            validated = []
            for prop in props:
                is_helper = isinstance(prop.get(name), AWSHelperFn)
                validated.append(name in prop or is_helper)
            return all(validated)

        def check_any(name, props):
            validated = []
            for prop in props:
                is_helper = isinstance(prop.get(name), AWSHelperFn)
                validated.append(name in prop and not is_helper)
            return any(validated)

        if isinstance(billing_mode, If):
            if check_any("ProvisionedThroughput", tput_props):
                raise ValueError(
                    "Table billing mode is per-request. "
                    "ProvisionedThroughput property is mutually exclusive"
                )
            return

        if billing_mode == "PROVISIONED":
            if not check_if_all("ProvisionedThroughput", tput_props):
                raise ValueError(
                    "Table billing mode is provisioned. "
                    "ProvisionedThroughput required if available"
                )
        elif billing_mode == "PAY_PER_REQUEST":
            if check_any("ProvisionedThroughput", tput_props):
                raise ValueError(
                    "Table billing mode is per-request. "
                    "ProvisionedThroughput property is mutually exclusive"
                )



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 35.1.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import double


class StreamConfiguration(AWSProperty):
    props = {
        "ClipboardMode": (str, True),
        "Ec2InstanceTypes": ([str], True),
        "MaxSessionLengthInMinutes": (double, False),
        "StreamingImageIds": ([str], True),
    }


class LaunchProfile(AWSObject):
    resource_type = "AWS::NimbleStudio::LaunchProfile"

    props = {
        "Description": (str, False),
        "Ec2SubnetIds": ([str], True),
        "LaunchProfileProtocolVersions": ([str], True),
        "Name": (str, True),
        "StreamConfiguration": (StreamConfiguration, True),
        "StudioComponentIds": ([str], True),
        "StudioId": (str, True),
        "Tags": (Tags, False),
    }


class StreamingImage(AWSObject):
    resource_type = "AWS::NimbleStudio::StreamingImage"

    props = {
        "Description": (str, False),
        "Ec2ImageId": (str, True),
        "Name": (str, True),
        "StudioId": (str, True),
        "Tags": (Tags, False),
    }


class StudioEncryptionConfiguration(AWSProperty):
    props = {
        "KeyArn": (str, False),
        "KeyType": (str, True),
    }


class Studio(AWSObject):
    resource_type = "AWS::NimbleStudio::Studio"

    props = {
        "AdminRoleArn": (str, True),
        "DisplayName": (str, True),
        "StudioEncryptionConfiguration": (StudioEncryptionConfiguration, False),
        "StudioName": (str, True),
        "Tags": (Tags, False),
        "UserRoleArn": (str, True),
    }


class ScriptParameterKeyValue(AWSProperty):
    props = {
        "Key": (str, False),
        "Value": (str, False),
    }


class ActiveDirectoryComputerAttribute(AWSProperty):
    props = {
        "Name": (str, False),
        "Value": (str, False),
    }


class ActiveDirectoryConfiguration(AWSProperty):
    props = {
        "ComputerAttributes": ([ActiveDirectoryComputerAttribute], False),
        "DirectoryId": (str, False),
        "OrganizationalUnitDistinguishedName": (str, False),
    }


class ComputeFarmConfiguration(AWSProperty):
    props = {
        "ActiveDirectoryUser": (str, False),
        "Endpoint": (str, False),
    }


class LicenseServiceConfiguration(AWSProperty):
    props = {
        "Endpoint": (str, False),
    }


class SharedFileSystemConfiguration(AWSProperty):
    props = {
        "Endpoint": (str, False),
        "FileSystemId": (str, False),
        "LinuxMountPoint": (str, False),
        "ShareName": (str, False),
        "WindowsMountDrive": (str, False),
    }


class StudioComponentConfiguration(AWSProperty):
    props = {
        "ActiveDirectoryConfiguration": (ActiveDirectoryConfiguration, False),
        "ComputeFarmConfiguration": (ComputeFarmConfiguration, False),
        "LicenseServiceConfiguration": (LicenseServiceConfiguration, False),
        "SharedFileSystemConfiguration": (SharedFileSystemConfiguration, False),
    }


class StudioComponentInitializationScript(AWSProperty):
    props = {
        "LaunchProfileProtocolVersion": (str, False),
        "Platform": (str, False),
        "RunContext": (str, False),
        "Script": (str, False),
    }


class StudioComponent(AWSObject):
    resource_type = "AWS::NimbleStudio::StudioComponent"

    props = {
        "Configuration": (StudioComponentConfiguration, False),
        "Description": (str, False),
        "Ec2SecurityGroupIds": ([str], False),
        "InitializationScripts": ([StudioComponentInitializationScript], False),
        "Name": (str, True),
        "ScriptParameters": ([ScriptParameterKeyValue], False),
        "StudioId": (str, True),
        "Subtype": (str, False),
        "Tags": (Tags, False),
        "Type": (str, True),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.7.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean


class Ledger(AWSObject):
    resource_type = "AWS::QLDB::Ledger"

    props = {
        "DeletionProtection": (boolean, False),
        "KmsKey": (str, False),
        "Name": (str, False),
        "PermissionsMode": (str, True),
        "Tags": (Tags, False),
    }


class KinesisConfiguration(AWSProperty):
    props = {
        "AggregationEnabled": (boolean, False),
        "StreamArn": (str, False),
    }


class Stream(AWSObject):
    resource_type = "AWS::QLDB::Stream"

    props = {
        "ExclusiveEndTime": (str, False),
        "InclusiveStartTime": (str, True),
        "KinesisConfiguration": (KinesisConfiguration, True),
        "LedgerName": (str, True),
        "RoleArn": (str, True),
        "StreamName": (str, True),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import (
    accelerator_ipaddresstype,
    boolean,
    double,
    endpointgroup_healthcheckprotocol,
    integer,
    listener_clientaffinity,
    listener_protocol,
)


class Accelerator(AWSObject):
    resource_type = "AWS::GlobalAccelerator::Accelerator"

    props = {
        "Enabled": (boolean, False),
        "IpAddresses": ([str], False),
        "IpAddressType": (accelerator_ipaddresstype, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class PortRange(AWSProperty):
    props = {
        "FromPort": (integer, True),
        "ToPort": (integer, True),
    }


class Listener(AWSObject):
    resource_type = "AWS::GlobalAccelerator::Listener"

    props = {
        "AcceleratorArn": (str, True),
        "ClientAffinity": (listener_clientaffinity, False),
        "PortRanges": ([PortRange], True),
        "Protocol": (listener_protocol, False),
    }


class EndpointConfiguration(AWSProperty):
    props = {
        "ClientIPPreservationEnabled": (boolean, False),
        "EndpointId": (str, True),
        "Weight": (integer, False),
    }


class EndpointGroup(AWSObject):
    resource_type = "AWS::GlobalAccelerator::EndpointGroup"

    props = {
        "EndpointConfigurations": ([EndpointConfiguration], False),
        "EndpointGroupRegion": (str, True),
        "HealthCheckIntervalSeconds": (integer, False),
        "HealthCheckPath": (str, False),
        "HealthCheckPort": (integer, False),
        "HealthCheckProtocol": (endpointgroup_healthcheckprotocol, False),
        "ListenerArn": (str, True),
        "ThresholdCount": (integer, False),
        "TrafficDialPercentage": (double, False),
    }



import troposphere.ec2 as ec2
from troposphere import Tags, Template

t = Template()

t.add_resource(
    ec2.ClientVpnEndpoint(
        "myClientVpnEndpoint",
        AuthenticationOptions=[
            ec2.ClientAuthenticationRequest(
                Type="directory-service-authentication",
                ActiveDirectory=ec2.DirectoryServiceAuthenticationRequest(
                    DirectoryId="d-926example"
                ),
            )
        ],
        ClientCidrBlock="10.0.0.0/22",
        ConnectionLogOptions=ec2.ConnectionLogOptions(Enabled=False),
        Description="My Client VPN Endpoint",
        DnsServers=["11.11.0.1"],
        ServerCertificateArn=(
            "arn:aws:acm:us-east-1:111122223333:certificate/"
            "12345678-1234-1234-1234-123456789012"
        ),
        TagSpecifications=[
            ec2.TagSpecifications(
                ResourceType="client-vpn-endpoint",
                Tags=Tags(Purpose="Production"),
            )
        ],
        TransportProtocol="udp",
    )
)

print(t.to_json())



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 4.3.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, integer


class ClearTimer(AWSProperty):
    props = {
        "TimerName": (str, False),
    }


class Firehose(AWSProperty):
    props = {
        "DeliveryStreamName": (str, False),
        "Separator": (str, False),
    }


class IotEvents(AWSProperty):
    props = {
        "InputName": (str, False),
    }


class IotTopicPublish(AWSProperty):
    props = {
        "MqttTopic": (str, False),
    }


class Lambda(AWSProperty):
    props = {
        "FunctionArn": (str, False),
    }


class ResetTimer(AWSProperty):
    props = {
        "TimerName": (str, False),
    }


class SetTimer(AWSProperty):
    props = {
        "Seconds": (integer, False),
        "TimerName": (str, False),
    }


class SetVariable(AWSProperty):
    props = {
        "Value": (str, False),
        "VariableName": (str, False),
    }


class Sns(AWSProperty):
    props = {
        "TargetArn": (str, False),
    }


class Sqs(AWSProperty):
    props = {
        "QueueUrl": (str, False),
        "UseBase64": (boolean, False),
    }


class Action(AWSProperty):
    props = {
        "ClearTimer": (ClearTimer, False),
        "Firehose": (Firehose, False),
        "IotEvents": (IotEvents, False),
        "IotTopicPublish": (IotTopicPublish, False),
        "Lambda": (Lambda, False),
        "ResetTimer": (ResetTimer, False),
        "SetTimer": (SetTimer, False),
        "SetVariable": (SetVariable, False),
        "Sns": (Sns, False),
        "Sqs": (Sqs, False),
    }


class Event(AWSProperty):
    props = {
        "Actions": ([Action], False),
        "Condition": (str, False),
        "EventName": (str, False),
    }


class OnEnter(AWSProperty):
    props = {
        "Events": ([Event], False),
    }


class OnExit(AWSProperty):
    props = {
        "Events": ([Event], False),
    }


class TransitionEvent(AWSProperty):
    props = {
        "Actions": ([Action], False),
        "Condition": (str, False),
        "EventName": (str, False),
        "NextState": (str, False),
    }


class OnInput(AWSProperty):
    props = {
        "Events": ([Event], False),
        "TransitionEvents": ([TransitionEvent], False),
    }


class State(AWSProperty):
    props = {
        "OnEnter": (OnEnter, False),
        "OnExit": (OnExit, False),
        "OnInput": (OnInput, False),
        "StateName": (str, False),
    }


class DetectorModelDefinition(AWSProperty):
    props = {
        "InitialStateName": (str, False),
        "States": ([State], False),
    }


class DetectorModel(AWSObject):
    resource_type = "AWS::IoTEvents::DetectorModel"

    props = {
        "DetectorModelDefinition": (DetectorModelDefinition, False),
        "DetectorModelDescription": (str, False),
        "DetectorModelName": (str, False),
        "Key": (str, False),
        "RoleArn": (str, False),
        "Tags": (Tags, False),
    }


class Attribute(AWSProperty):
    props = {
        "JsonPath": (str, False),
    }


class InputDefinition(AWSProperty):
    props = {
        "Attributes": ([Attribute], False),
    }


class Input(AWSObject):
    resource_type = "AWS::IoTEvents::Input"

    props = {
        "InputDefinition": (InputDefinition, False),
        "InputDescription": (str, False),
        "InputName": (str, False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2020, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import AWSObject, AWSProperty, Tags
from .validators import boolean, json_checker


class IEMap(AWSProperty):
    props = {
        "ACCOUNT": ([str], False),
        "ORGUNIT": ([str], False),
    }


class Policy(AWSObject):
    resource_type = "AWS::FMS::Policy"

    props = {
        "DeleteAllPolicyResources": (boolean, False),
        "ExcludeMap": (IEMap, False),
        "ExcludeResourceTags": (boolean, True),
        "IncludeMap": (IEMap, False),
        "PolicyName": (str, True),
        "RemediationEnabled": (boolean, True),
        "ResourceTags": (Tags, False),
        "ResourceType": (str, True),
        "ResourceTypeList": ([str], True),
        "SecurityServicePolicyData": (json_checker, True),
        "Tags": (Tags, False),
    }


class NotificationChannel(AWSObject):
    resource_type = "AWS::FMS::NotificationChannel"

    props = {
        "SnsRoleName": (str, True),
        "SnsTopicArn": (str, True),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.8.0


from . import AWSObject, AWSProperty
from .validators import boolean, integer


class Cluster(AWSObject):
    resource_type = "AWS::Route53RecoveryControl::Cluster"

    props = {
        "Name": (str, False),
    }


class ControlPanel(AWSObject):
    resource_type = "AWS::Route53RecoveryControl::ControlPanel"

    props = {
        "ClusterArn": (str, False),
        "Name": (str, True),
    }


class RoutingControl(AWSObject):
    resource_type = "AWS::Route53RecoveryControl::RoutingControl"

    props = {
        "ClusterArn": (str, False),
        "ControlPanelArn": (str, False),
        "Name": (str, True),
    }


class AssertionRule(AWSProperty):
    props = {
        "AssertedControls": ([str], True),
        "WaitPeriodMs": (integer, True),
    }


class GatingRule(AWSProperty):
    props = {
        "GatingControls": ([str], True),
        "TargetControls": ([str], True),
        "WaitPeriodMs": (integer, True),
    }


class RuleConfig(AWSProperty):
    props = {
        "Inverted": (boolean, True),
        "Threshold": (integer, True),
        "Type": (str, True),
    }


class SafetyRule(AWSObject):
    resource_type = "AWS::Route53RecoveryControl::SafetyRule"

    props = {
        "AssertionRule": (AssertionRule, False),
        "ControlPanelArn": (str, True),
        "GatingRule": (GatingRule, False),
        "Name": (str, True),
        "RuleConfig": (RuleConfig, True),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 39.1.0


from . import AWSObject
from .validators import boolean


class ReportDefinition(AWSObject):
    resource_type = "AWS::CUR::ReportDefinition"

    props = {
        "AdditionalArtifacts": ([str], False),
        "AdditionalSchemaElements": ([str], False),
        "BillingViewArn": (str, False),
        "Compression": (str, True),
        "Format": (str, True),
        "RefreshClosedReports": (boolean, True),
        "ReportName": (str, True),
        "ReportVersioning": (str, True),
        "S3Bucket": (str, True),
        "S3Prefix": (str, True),
        "S3Region": (str, True),
        "TimeUnit": (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 18.6.0


from troposphere import Tags

from . import AWSObject, AWSProperty


class Channel(AWSProperty):
    props = {
        "channelId": (str, False),
        "channelUri": (str, True),
    }


class ProfilingGroup(AWSObject):
    resource_type = "AWS::CodeGuruProfiler::ProfilingGroup"

    props = {
        "AgentPermissions": (dict, False),
        "AnomalyDetectionNotificationConfiguration": ([Channel], False),
        "ComputePlatform": (str, False),
        "ProfilingGroupName": (str, True),
        "Tags": (Tags, False),
    }



#!/usr/bin/python

from troposphere import Base64, Join, Sub


def from_file(filepath, delimiter="", blanklines=False):
    """Imports userdata from a file.

    :type filepath: string

    :param filepath  The absolute path to the file.

    :type delimiter: string

    :param: delimiter  Delimiter to use with the troposphere.Join().

    :type blanklines: boolean

    :param blanklines  If blank lines should be ignored

    rtype: troposphere.Base64
    :return The base64 representation of the file.
    """
    data = []

    try:
        with open(filepath, "r") as f:
            for line in f:
                if blanklines and line.strip("\n\r ") == "":
                    continue

                data.append(line)
    except IOError:
        raise IOError("Error opening or reading file: {}".format(filepath))

    return Base64(Join(delimiter, data))


def from_file_sub(filepath):
    """Imports userdata from a file, using Sub for replacing inline variables such as ${AWS::Region}

    :type filepath: string

    :param filepath  The absolute path to the file.

    rtype: troposphere.Base64
    :return The base64 representation of the file.
    """

    try:
        with open(filepath, "rt") as f:
            data = f.read()
            return Base64(Sub(data))
    except IOError:
        raise IOError("Error opening or reading file: {}".format(filepath))



# Copyright (c) 2012-2018, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.

from . import AWSObject, AWSProperty


class Overrides(AWSProperty):
    props = {
        "Manifest": (dict, False),
    }


class AuthenticationConfiguration(AWSProperty):
    props = {
        "ClientId": (str, True),
        "ClientSecret": (str, True),
        "RefreshToken": (str, True),
    }


class SkillPackage(AWSProperty):
    props = {
        "Overrides": (Overrides, False),
        "S3Bucket": (str, True),
        "S3BucketRole": (str, False),
        "S3Key": (str, True),
        "S3ObjectVersion": (str, False),
    }


class Skill(AWSObject):
    resource_type = "Alexa::ASK::Skill"

    props = {
        "AuthenticationConfiguration": (AuthenticationConfiguration, True),
        "SkillPackage": (SkillPackage, True),
        "VendorId": (str, True),
    }



# Copyright (c) 2012-2019, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 3.3.0


from troposphere import Tags

from . import AWSObject, AWSProperty

VALID_HOMEDIRECTORY_TYPE = ("LOGICAL", "PATH")


def validate_homedirectory_type(homedirectory_type):
    """Validate HomeDirectoryType for User"""

    if homedirectory_type not in VALID_HOMEDIRECTORY_TYPE:  # NOQA
        raise ValueError(
            "User HomeDirectoryType must be one of: %s"
            % ", ".join(VALID_HOMEDIRECTORY_TYPE)  # NOQA
        )
    return homedirectory_type


class EndpointDetails(AWSProperty):
    props = {
        "AddressAllocationIds": ([str], False),
        "SecurityGroupIds": ([str], False),
        "SubnetIds": ([str], False),
        "VpcEndpointId": (str, False),
        "VpcId": (str, False),
    }


class IdentityProviderDetails(AWSProperty):
    props = {
        "InvocationRole": (str, True),
        "Url": (str, True),
    }


class ProtocolDetails(AWSProperty):
    props = {
        "PassiveIp": (str, False),
    }


class Server(AWSObject):
    resource_type = "AWS::Transfer::Server"

    props = {
        "Certificate": (str, False),
        "Domain": (str, False),
        "EndpointDetails": (EndpointDetails, False),
        "EndpointType": (str, False),
        "IdentityProviderDetails": (IdentityProviderDetails, False),
        "IdentityProviderType": (str, False),
        "LoggingRole": (str, False),
        "ProtocolDetails": (ProtocolDetails, False),
        "Protocols": ([str], False),
        "SecurityPolicyName": (str, False),
        "Tags": (Tags, False),
    }


class HomeDirectoryMapEntry(AWSProperty):
    props = {
        "Entry": (str, True),
        "Target": (str, True),
    }


class User(AWSObject):
    resource_type = "AWS::Transfer::User"

    props = {
        "HomeDirectory": (str, False),
        "HomeDirectoryMappings": ([HomeDirectoryMapEntry], False),
        "HomeDirectoryType": (validate_homedirectory_type, False),
        "Policy": (str, False),
        "Role": (str, True),
        "ServerId": (str, True),
        "SshPublicKeys": ([str], False),
        "Tags": (Tags, False),
        "UserName": (str, True),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***
# Resource specification version: 31.0.0


from troposphere import Tags

from . import AWSObject, AWSProperty
from .validators import boolean, double


class AnalysisError(AWSProperty):
    props = {
        "Message": (str, False),
        "Type": (str, False),
    }


class DataSetReference(AWSProperty):
    props = {
        "DataSetArn": (str, True),
        "DataSetPlaceholder": (str, True),
    }


class AnalysisSourceTemplate(AWSProperty):
    props = {
        "Arn": (str, True),
        "DataSetReferences": ([DataSetReference], True),
    }


class AnalysisSourceEntity(AWSProperty):
    props = {
        "SourceTemplate": (AnalysisSourceTemplate, False),
    }


class DateTimeParameter(AWSProperty):
    props = {
        "Name": (str, True),
        "Values": ([str], True),
    }


class DecimalParameter(AWSProperty):
    props = {
        "Name": (str, True),
        "Values": ([double], True),
    }


class IntegerParameter(AWSProperty):
    props = {
        "Name": (str, True),
        "Values": ([double], True),
    }


class StringParameter(AWSProperty):
    props = {
        "Name": (str, True),
        "Values": ([str], True),
    }


class Parameters(AWSProperty):
    props = {
        "DateTimeParameters": ([DateTimeParameter], False),
        "DecimalParameters": ([DecimalParameter], False),
        "IntegerParameters": ([IntegerParameter], False),
        "StringParameters": ([StringParameter], False),
    }


class ResourcePermission(AWSProperty):
    props = {
        "Actions": ([str], True),
        "Principal": (str, True),
    }


class Analysis(AWSObject):
    resource_type = "AWS::QuickSight::Analysis"

    props = {
        "AnalysisId": (str, True),
        "AwsAccountId": (str, True),
        "Errors": ([AnalysisError], False),
        "Name": (str, False),
        "Parameters": (Parameters, False),
        "Permissions": ([ResourcePermission], False),
        "SourceEntity": (AnalysisSourceEntity, False),
        "Tags": (Tags, False),
        "ThemeArn": (str, False),
    }


class AdHocFilteringOption(AWSProperty):
    props = {
        "AvailabilityStatus": (str, False),
    }


class ExportToCSVOption(AWSProperty):
    props = {
        "AvailabilityStatus": (str, False),
    }


class SheetControlsOption(AWSProperty):
    props = {
        "VisibilityState": (str, False),
    }


class DashboardPublishOptions(AWSProperty):
    props = {
        "AdHocFilteringOption": (AdHocFilteringOption, False),
        "ExportToCSVOption": (ExportToCSVOption, False),
        "SheetControlsOption": (SheetControlsOption, False),
    }


class DashboardSourceTemplate(AWSProperty):
    props = {
        "Arn": (str, True),
        "DataSetReferences": ([DataSetReference], True),
    }


class DashboardSourceEntity(AWSProperty):
    props = {
        "SourceTemplate": (DashboardSourceTemplate, False),
    }


class Dashboard(AWSObject):
    resource_type = "AWS::QuickSight::Dashboard"

    props = {
        "AwsAccountId": (str, True),
        "DashboardId": (str, True),
        "DashboardPublishOptions": (DashboardPublishOptions, False),
        "Name": (str, False),
        "Parameters": (Parameters, False),
        "Permissions": ([ResourcePermission], False),
        "SourceEntity": (DashboardSourceEntity, False),
        "Tags": (Tags, False),
        "ThemeArn": (str, False),
        "VersionDescription": (str, False),
    }


class TemplateSourceAnalysis(AWSProperty):
    props = {
        "Arn": (str, True),
        "DataSetReferences": ([DataSetReference], True),
    }


class TemplateSourceTemplate(AWSProperty):
    props = {
        "Arn": (str, True),
    }


class TemplateSourceEntity(AWSProperty):
    props = {
        "SourceAnalysis": (TemplateSourceAnalysis, False),
        "SourceTemplate": (TemplateSourceTemplate, False),
    }


class Template(AWSObject):
    resource_type = "AWS::QuickSight::Template"

    props = {
        "AwsAccountId": (str, True),
        "Name": (str, False),
        "Permissions": ([ResourcePermission], False),
        "SourceEntity": (TemplateSourceEntity, False),
        "Tags": (Tags, False),
        "TemplateId": (str, True),
        "VersionDescription": (str, False),
    }


class DataColorPalette(AWSProperty):
    props = {
        "Colors": ([str], False),
        "EmptyFillColor": (str, False),
        "MinMaxGradient": ([str], False),
    }


class GutterStyle(AWSProperty):
    props = {
        "Show": (boolean, False),
    }


class MarginStyle(AWSProperty):
    props = {
        "Show": (boolean, False),
    }


class TileLayoutStyle(AWSProperty):
    props = {
        "Gutter": (GutterStyle, False),
        "Margin": (MarginStyle, False),
    }


class BorderStyle(AWSProperty):
    props = {
        "Show": (boolean, False),
    }


class TileStyle(AWSProperty):
    props = {
        "Border": (BorderStyle, False),
    }


class SheetStyle(AWSProperty):
    props = {
        "Tile": (TileStyle, False),
        "TileLayout": (TileLayoutStyle, False),
    }


class Font(AWSProperty):
    props = {
        "FontFamily": (str, False),
    }


class Typography(AWSProperty):
    props = {
        "FontFamilies": ([Font], False),
    }


class UIColorPalette(AWSProperty):
    props = {
        "Accent": (str, False),
        "AccentForeground": (str, False),
        "Danger": (str, False),
        "DangerForeground": (str, False),
        "Dimension": (str, False),
        "DimensionForeground": (str, False),
        "Measure": (str, False),
        "MeasureForeground": (str, False),
        "PrimaryBackground": (str, False),
        "PrimaryForeground": (str, False),
        "SecondaryBackground": (str, False),
        "SecondaryForeground": (str, False),
        "Success": (str, False),
        "SuccessForeground": (str, False),
        "Warning": (str, False),
        "WarningForeground": (str, False),
    }


class ThemeConfiguration(AWSProperty):
    props = {
        "DataColorPalette": (DataColorPalette, False),
        "Sheet": (SheetStyle, False),
        "Typography": (Typography, False),
        "UIColorPalette": (UIColorPalette, False),
    }


class Theme(AWSObject):
    resource_type = "AWS::QuickSight::Theme"

    props = {
        "AwsAccountId": (str, True),
        "BaseThemeId": (str, False),
        "Configuration": (ThemeConfiguration, False),
        "Name": (str, False),
        "Permissions": ([ResourcePermission], False),
        "Tags": (Tags, False),
        "ThemeId": (str, True),
        "VersionDescription": (str, False),
    }
